/************************************************************************
 * This library is free software; you can redistribute it and/or	*
 * modify it under the terms of the GNU Library General Public		*
 * License as published by the Free Software Foundation; either		*
 * version 2 of the License, or (at your option) any later version.	*
 * 									*
 * This library is distributed in the hope that it will be useful,	*
 * but WITHOUT ANY WARRANTY; without even the implied warranty of	*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*
 * Library General Public License for more details.			*
 * 									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the		*
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,		*
 * Boston, MA  02111-1307, USA.						*
 * 									*
 * The GNU Library General Public License file is included in		*
 * Online-Documentation							*
 *									*
 * GNU's Homepage - http://www.gnu.ai.mit.edu/home.html			*
 *									*
 * Questions & comments referring to *FLIP++: please contact us:	*
 *									*
 * Snail-Mail:								*
 *	Dr. Wolfgang SLANY						*
 *	Database and Expert System Group (E184-2)     			*
 *	Institute of Information Systems               			*
 *	Vienna University of Technology					*
 *	Paniglgasse 16, A-1040 Vienna, AUSTRIA				*
 * Telephone:								*
 *	+43-1-58801-6123     (area-code 1 maybe changed to 201 in 1998) *
 * FAX:									*
 *	+43-1-5055304	     (area-code 1 maybe changed to 201 in 1998) *
 * eMail:								*
 *	sflip@dbai.tuwien.ac.at						*
 * WWW:									*
 *	http://www.dbai.tuwien.ac.at/proj/StarFLIP/			*
 *                                                                      *
 * $Id: cflplist.cpp,v 1.4 1997/06/24 17:28:37 reichel Exp $ *
 ***********************************************************************/


/********************************************************/
/*  modul       ?       cflplist.cpp                    */
/*------------------------------------------------------*/
/*  project     ?       C O N F L I P + +               */
/********************************************************/
/*  by whom     ?       Gerhilde Egghart                */
/*  with whom   ?       Wolfgang Slany                  */
/*  when        ?       1993/1994                       */
/*  what        ?       modelling of constraints        */
/*  why         ?       Praktikum I + II                */
/*  how         ?       C++                             */
/********************************************************/
/*              ...to be continued...                   */
/*------------------------------------------------------*/
/*  by whom     ?       Georg Fischer                   */
/*  with whom   ?       Wolfgang Slany                  */
/*  when        ?       1994                            */
/*  why         ?       Praktikum I                     */
/*  what done   ?       redesigned and                  */
/*                      making much more sturdy         */
/********************************************************/


#include <stdlib.h>
#include <unistd.h>
#include <fstream.h>
#include <string.h>

//#include "cflperr.h"
#include "cflplist.h"

//extern conflip_error_t conflip_error;


GenList::GenList(void)
{
  init();
}


GenList::GenList(GenList & const source)
{
  init();
  copy(source);
}


GenList::GenList(GenSled & const source)
{
  init();
  copy(*source.list_ptr);
}


GenList::~GenList(void)
{
  delete(root);
}


void GenList::clear(void)
{
  Node1d *help=root->succ;
  while(help!=root)
  {
    del(help->item);
    help=help->succ;
    delete(help->pred);
  }
  help->succ=help;
  help->pred=help;
  flag=lf_init;
  no_of_items=0;
}


void GenList::copy(GenList & const source)
{
  Node1d *help1=source.root->succ;
  Node1d *help2=root;
  while(help1!=source.root)
  {
    help2->succ=new(Node1d);
    cpy(help2->succ->item,help1->item);
    help2->succ->pred=help2;
    help2=help2->succ;
    help1=help1->succ;
  }
  help2->succ=root;
  root->pred=help2;
  flag=source.flag;
  no_of_items=source.no_of_items;
}


void GenList::init(void)
{
  root=new(Node1d);
  root->pred=root;
  root->succ=root;
  flag=lf_init;
  no_of_items=0;
}


bool GenList::insert(Node1d *node,void *info,InfoCopy f)
{
  Node1d *help;
  if((help=new(Node1d))!=NULL)
  {
    if(f==copy_when_insert)
      cpy(help->item,info);
    else
      help->item=info;
    help->succ=node->succ;
    node->succ->pred=help;
    help->pred=node;
    node->succ=help;
    no_of_items++;
    return(true);
  }
  else
  {
    // conflip_error.set(conflip_error_t::memory_allocation_failed);
    return(false);
  }
}


bool GenList::kick_out(Node1d *node)
{
  if(node!=root)
  {
    del(node->item);
    node->pred->succ=node->succ;
    node->succ->pred=node->pred;
    delete(node);
    no_of_items--;
    return(true);
  }
  else
  {
    // conflip_error.set(conflip_error_t::list_nothing_to_delete);
    return(false);
  }
}


GenList &GenList::operator =(GenList & const par)
{
  clear();
  copy(par);
  return(*this);
}


GenList &GenList::operator =(GenSled & const par)
{
  clear();
  copy(*par.list_ptr);
  return(*this);
}


bool GenList::operator ==(GenList & const par)
{
  GenSled help1,help2;
  if(no_of_items==par.no_of_items)
  {
    help1=this;
    help2=par;
    for(int i=1;i<=no_of_items;i++)
      if(!cmp(help1++,help2++))
        return(false);
    return(true);
  }
  return(false);
}


bool GenList::week_equal(GenList & const par)
{
  int i;
  Node1d *help1;
  Node1d *help2;
  help1=root->succ;
  for(i=1;i<=no_of_items;i++)
    if((help2=par.lookup(key(help1->item)))!=NULL)
      if(cmp(help1->item,help2->item))
        help1=help1->succ;
      else
        return(false);
    else
      return(false);
  help1=par.root->succ;
  for(i=1;i<=par.no_of_items;i++)
    if((help2=lookup(key(help1->item)))!=NULL)
      if(par.cmp(help1->item,help2->item))
        help1=help1->succ;
      else
        return(false);
    else
      return(false);
  return(true);
}


bool GenList::read(ifstream &infile)
{
  int to_read;
  int temp_flag;
  void *temp;
  clear();
  if(infile>>to_read>>temp_flag)
  {
    for(int i=1;i<=to_read;i++)
      if(rd(infile,temp))
      {
        if(!add_quick(temp,no_copy_when_insert))
        {
          clear();
          return(false);
        }
      }
      else
      {
        clear();
        return(false);
      }
    flag=(GenListFlag)temp_flag;
    return(true);
  }
  else
  {
    // conflip_error.set(conflip_error_t::file_read_error);
    return(false);
  }
  return(false);
}


bool GenList::write(ofstream &outfile)
{
  GenSled help=this;
  if(outfile<<no_of_items<<' '<<(int)flag<<' ')
  {
    for(int i=1;i<=no_of_items;i++)
      if(wr(outfile,help++))
        continue;
      else
      {
        // conflip_error.set(conflip_error_t::file_write_error);
        return(false);
      }
    return(true);
  }
  else
  {
    // conflip_error.set(conflip_error_t::file_write_error);
    return(false);
  }
}


bool GenList::add(void *info,InfoCopy f)
{
  Node1d *help;
  if(lookup(key(info))==NULL)
    return(add_quick(info,f));
  else
  {
    // conflip_error.set(conflip_error_t::name_does_already_exist,0,(char *)key(info));
    return(false);
  }
}


bool GenList::add_quick(void *info,InfoCopy f)
{
  return(insert(root->pred,info,f));
}


bool GenList::kick_out(void *nam)
{
  Node1d *help;
  if((help=lookup(nam))!=NULL)
    return(kick_out(help));
  // conflip_error.set(conflip_error_t::name_does_not_exist,0,(char *)nam);
  return(false);
}


Node1d *GenList::lookup(void *nam,Node1d *node,int &count)
{
  while(node!=root&&!look(nam,node->item))
  {
    node=node->succ;
    count++;
  }
  if(node!=root)
    return(node);
  count=0;
  return(NULL);
}


Node1d *GenList::lookup(void *nam,Node1d *node)
{
  int dummy;
  return(lookup(nam,node,dummy));
}


Node1d *GenList::lookup(void *nam)
{
  return(lookup(nam,root->succ));
}


void GenList::show(void)
{
  Node1d *help=root->succ;
  ofstream output(dup(1));
  output<<"LIST::SHOW"<<endl;
  write(output);
  output<<endl<<"END OF LIST::SHOW"<<endl;
  output.close();
}


GenList::GenListFlag GenList::get_flags(void)
{
  return(flag);
}


void GenList::set_flags(int set)
{
  flag=(GenListFlag)(flag|set);
}


void GenList::clr_flags(int set)
{
  flag=(GenListFlag)(flag&~set);
}


//-------------------------GenSled--------------------------------


GenSled::GenSled(void)
{
  ptr=NULL;
  list_ptr=NULL;
  index=0;
}


GenSled::GenSled(GenList & const l)
{
  ptr=l.root->succ;
  list_ptr=&l;
  if(ptr!=l.root)
    index=1;
  else
    index=0;
}


GenSled::GenSled(GenList * const l)
{
  ptr=l->root->succ;
  list_ptr=l;
  if(ptr!=l->root)
    index=1;
  else
    index=0;
}


void *GenSled::operator ++(void)
{
  ptr=ptr->succ;
  index=(index+1)%(list_ptr->no_of_items+1);
  if(ptr!=list_ptr->root)
    return(ptr->item);
  return(NULL);
}


void *GenSled::operator ++(int)
{
  Node1d *temp=ptr;
  ptr=ptr->succ;
  index=(index+1)%(list_ptr->no_of_items+1);
  if(temp!=list_ptr->root)
    return(temp->item);
  return(NULL);
}


void *GenSled::operator --(void)
{
  ptr=ptr->pred;
  index=(index+list_ptr->no_of_items)%(list_ptr->no_of_items+1);
  if(ptr!=list_ptr->root)
    return(ptr->item);
  return(NULL);
}


void *GenSled::operator --(int)
{
  Node1d *temp=ptr;
  ptr=ptr->pred;
  index=(index+list_ptr->no_of_items)%(list_ptr->no_of_items+1);
  if(temp!=list_ptr->root)
    return(temp->item);
  return(NULL);
}


void *GenSled::operator [](int pos)
{
  if(pos>0)
    if(list_ptr->no_of_items!=0)
    {
      int help=index;
      index=(((pos-1)%(list_ptr->no_of_items))+1);
      pos=help-index;
      if(abs(help-index)>(list_ptr->no_of_items/2))
        if(pos<0)
        {
          pos=list_ptr->no_of_items+pos+1;
          while(pos-->0)
            ptr=ptr->pred;
        }
        else
        {
          pos=list_ptr->no_of_items-pos+1;
          while(pos-->0)
            ptr=ptr->succ;
        }
      else
        if(pos>=0)
          while(pos-->0)
            ptr=ptr->pred;
        else
          while(pos++<0)
            ptr=ptr->succ;
    }
    else
    {
      ptr=list_ptr->root;
      index=0;
    }
  else
  {
    ptr=list_ptr->root;
    index=0;
  }
  return(operator()());
}

void *GenSled::get_ptr(int pos)	//	3.95 hendry
{
  if(pos>0)
    if(list_ptr->no_of_items!=0)
    {
      int help=index;
      index=(((pos-1)%(list_ptr->no_of_items))+1);
      pos=help-index;
      if(abs(help-index)>(list_ptr->no_of_items/2))
        if(pos<0)
        {
          pos=list_ptr->no_of_items+pos+1;
          while(pos-->0)
            ptr=ptr->pred;
        }
        else
        {
          pos=list_ptr->no_of_items-pos+1;
          while(pos-->0)
            ptr=ptr->succ;
        }
      else
        if(pos>=0)
          while(pos-->0)
            ptr=ptr->pred;
        else
          while(pos++<0)
            ptr=ptr->succ;
    }
    else
    {
      ptr=list_ptr->root;
      index=0;
    }
  else
  {
    ptr=list_ptr->root;
    index=0;
  }
  return(ptr);
}


void *GenSled::operator +=(int s)
{
  return(operator[](index+s));
}


void *GenSled::operator -=(int s)
{
  return(operator[](index-s));
}


void *GenSled::operator ()(void)
{
  if(ptr!=list_ptr->root)
    return(ptr->item);
  return(NULL);
}


bool GenSled::isvalid(void)
{
  return(ptr!=list_ptr->root?true:false);
}


bool GenSled::islast(void)
{
  return(ptr==list_ptr->root->pred?true:false);
}


bool GenSled::isfirst(void)
{
  return(ptr==list_ptr->root->succ?true:false);
}


void *GenSled::first(void)
{
  ptr=list_ptr->root->succ;
  if(ptr!=list_ptr->root)
    index=1;
  else
    index=0;
  return(operator()());
}


void *GenSled::last(void)
{
  ptr=list_ptr->root->pred;
  if(ptr!=list_ptr->root)
    index=list_ptr->no_of_items;
  else
    index=0;
  return(operator()());
}


int GenSled::position(void)
{
  return(index);
}


void *GenSled::lookup(void *l)
{
  Node1d *temp;
  if((temp=list_ptr->lookup(l,ptr,index))!=NULL)
    ptr=temp;
  else
    ptr=list_ptr->root;
  return(operator()());
}


bool GenSled::insert(void *info,InfoCopy f)
{
  if(list_ptr->flag&GenList::lf_insert)
  {
    // conflip_error.set(conflip_error_t::list_insert_not_permitted,index,(char *)list_ptr->key(info));
    return(false);
  }
  else
    if(list_ptr->insert(ptr,info,f))
    {
      ptr=ptr->succ;
      index++;								//#fi 941122
      return(true);
    }
    else
      return(false);
}


bool GenSled::add(void *info,InfoCopy f)
{
  if(list_ptr->flag&GenList::lf_insert)
  {
    // conflip_error.set(conflip_error_t::list_insert_not_permitted,index,(char *)list_ptr->key(info));
    return(false);
  }
  else
    return(list_ptr->add(info,f));
}


bool GenSled::add_quick(void *info,InfoCopy f)
{
  if(list_ptr->flag&GenList::lf_insert)
  {
    // conflip_error.set(conflip_error_t::list_insert_not_permitted,index,(char *)list_ptr->key(info));
    return(false);
  }
  else
    return(list_ptr->add_quick(info,f));
}


bool GenSled::kick_out(void)
{
  Node1d *temp;
  if(list_ptr->flag&GenList::lf_kick_out)
  {
    // conflip_error.set(conflip_error_t::list_delete_not_permitted,index,(char *)list_ptr->key(ptr->item));
    return(false);
  }
  else
  {
    temp=ptr->pred;
    if(list_ptr->kick_out(ptr))
    {
      ptr=temp;
      index--;								//#fi 941122
      return(true);
    }
    else
      return(false);
  }
}

int GenSled::get_how_full(void)
{
  return(list_ptr->no_of_items);
}

