/************************************************************************
 * This library is free software; you can redistribute it and/or	*
 * modify it under the terms of the GNU Library General Public		*
 * License as published by the Free Software Foundation; either		*
 * version 2 of the License, or (at your option) any later version.	*
 * 									*
 * This library is distributed in the hope that it will be useful,	*
 * but WITHOUT ANY WARRANTY; without even the implied warranty of	*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*
 * Library General Public License for more details.			*
 * 									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the		*
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,		*
 * Boston, MA  02111-1307, USA.						*
 * 									*
 * The GNU Library General Public License file is included in		*
 * Online-Documentation							*
 *									*
 * GNU's Homepage - http://www.gnu.ai.mit.edu/home.html			*
 *									*
 * Questions & comments referring to *FLIP++: please contact us:	*
 *									*
 * Snail-Mail:								*
 *	Dr. Wolfgang SLANY						*
 *	Database and Expert System Group (E184-2)     			*
 *	Institute of Information Systems               			*
 *	Vienna University of Technology					*
 *	Paniglgasse 16, A-1040 Vienna, AUSTRIA				*
 * Telephone:								*
 *	+43-1-58801-6123     (area-code 1 maybe changed to 201 in 1998) *
 * FAX:									*
 *	+43-1-5055304	     (area-code 1 maybe changed to 201 in 1998) *
 * eMail:								*
 *	sflip@dbai.tuwien.ac.at						*
 * WWW:									*
 *	http://www.dbai.tuwien.ac.at/proj/StarFLIP/			*
 *                                                                      *
 * $Id: cflplist.h,v 1.4 1997/06/24 17:28:39 reichel Exp $ *
 ***********************************************************************/


/********************************************************/
/*  modul       ?       cflplist.h                      */
/*------------------------------------------------------*/
/*  project     ?       C O N F L I P + +               */
/********************************************************/
/*  by whom     ?       Gerhilde Egghart                */
/*  with whom   ?       Wolfgang Slany                  */
/*  when        ?       1993/1994                       */
/*  what        ?       modelling of constraints        */
/*  why         ?       Praktikum I + II                */
/*  how         ?       C++                             */
/********************************************************/
/*              ...to be continued...                   */
/*------------------------------------------------------*/
/*  by whom     ?       Georg Fischer                   */
/*  with whom   ?       Wolfgang Slany                  */
/*  when        ?       1994                            */
/*  why         ?       Praktikum I                     */
/*  what done   ?       redesigned and                  */
/*                      making much more sturdy         */
/********************************************************/


#ifndef _CFLPLIST
#define _CFLPLIST

#include <fstream.h>

#include "bool.h"

class Node1d
{
  friend class GenSled;
  friend class GenList;
protected:
  void *item;
  Node1d *pred;
  Node1d *succ;
public:
  Node1d(void) { item=NULL; }
};


enum InfoCopy { no_copy_when_insert=0,
                copy_when_insert=1 };

class GenList
{
  friend class GenSled;
public:
  enum GenListFlag { lf_init=0,
                     lf_insert=1,
                     lf_kick_out=2 };
protected:
  int no_of_items;
  GenListFlag flag;
  Node1d *root;
  virtual void del(void *) {}
  virtual void *key(void *) { return(NULL); }
  virtual bool look(void *,void *) { return(false); }
  virtual bool cmp(void *,void *) { return(false); }
  virtual void cpy(void *&,void *) {}
  virtual bool rd(ifstream &,void *&) { return(false); }
  virtual bool wr(ofstream &,void *) { return(false); }
  void init(void);
  virtual bool insert(Node1d *,void *,InfoCopy =copy_when_insert);
  virtual bool kick_out(Node1d *);
  virtual Node1d *lookup(void *,Node1d *,int &);
  Node1d *lookup(void *,Node1d *);
  Node1d *lookup(void *);
  bool add(void *,InfoCopy =copy_when_insert);
  bool add_quick(void *,InfoCopy =copy_when_insert);
  bool kick_out(void *);
  void *get_info(Node1d *n) { return((n!=root&&n!=NULL)?(n->item):NULL); }
public:
  GenList(void);
  GenList(GenList & const);
  GenList(GenSled & const);
  ~GenList(void);
  void clear(void);
  void copy(GenList & const);

  GenList &operator =(GenList & const);
  GenList &operator =(GenSled & const);
  bool operator ==(GenList & const);
  bool week_equal(GenList & const);

  bool read(ifstream &);
  bool write(ofstream &);

  int get_how_full(void) { return(no_of_items); }
  GenListFlag get_flags(void);
  void set_flags(int);
  void clr_flags(int);
  void show(void);
};

class GenSled
{
  friend class GenList;
protected:
  Node1d *ptr;
  GenList *list_ptr;
  int index;
public:
  GenSled(void);
  GenSled(GenList & const);
  GenSled(GenList * const);
  void *operator ++(void);
  void *operator ++(int);
  void *operator --(void);
  void *operator --(int);
  void *operator [](int);
  void *operator +=(int);
  void *operator -=(int);
  void *operator ()(void);
  bool isvalid(void);
  bool islast(void);
  bool isfirst(void);
  void *first(void);
  void *last(void);
  int position(void);
  void *lookup(void *);
  bool insert(void *,InfoCopy =copy_when_insert);
  bool add(void *,InfoCopy =copy_when_insert);
  bool add_quick(void *,InfoCopy =copy_when_insert);
  bool kick_out(void);
  int get_how_full(void);
  //	!!! DomFLIP++ (2.95)
  void *get_ptr(int);	

};


#endif
