/************************************************************************
 * This library is free software; you can redistribute it and/or	*
 * modify it under the terms of the GNU Library General Public		*
 * License as published by the Free Software Foundation; either		*
 * version 2 of the License, or (at your option) any later version.	*
 * 									*
 * This library is distributed in the hope that it will be useful,	*
 * but WITHOUT ANY WARRANTY; without even the implied warranty of	*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*
 * Library General Public License for more details.			*
 * 									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the		*
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,		*
 * Boston, MA  02111-1307, USA.						*
 * 									*
 * The GNU Library General Public License file is included in		*
 * Online-Documentation							*
 *									*
 * GNU's Homepage - http://www.gnu.ai.mit.edu/home.html			*
 *									*
 * Questions & comments referring to *FLIP++: please contact us:	*
 *									*
 * Snail-Mail:								*
 *	Dr. Wolfgang SLANY						*
 *	Database and Expert System Group (E184-2)     			*
 *	Institute of Information Systems               			*
 *	Vienna University of Technology					*
 *	Paniglgasse 16, A-1040 Vienna, AUSTRIA				*
 * Telephone:								*
 *	+43-1-58801-6123     (area-code 1 maybe changed to 201 in 1998) *
 * FAX:									*
 *	+43-1-5055304	     (area-code 1 maybe changed to 201 in 1998) *
 * eMail:								*
 *	sflip@dbai.tuwien.ac.at						*
 * WWW:									*
 *	http://www.dbai.tuwien.ac.at/proj/StarFLIP/			*
 *                                                                      *
 * $Id: domagg.cpp,v 1.4 1997/06/24 17:28:49 reichel Exp $ *
 ***********************************************************************/


// domagg.cpp

#include "domagg.h"

char illegal_chr_in_agg_name[]="";
char default_agg_name[]="AGGR%.5u";


/*-------------------------------------------------------------------*/
/* Aggregate                                                   	     */
/*-------------------------------------------------------------------*/
// ******************************************************************
bool check_agg(char *n)
{
  char *t;
  int len=strlen(n);
  if((len<1)||(len>(LENGTH_OF_NAMES-1)))
  {
    domflip_error.set(domflip_error_t::illegal_agg_name_len, (LENGTH_OF_NAMES-1), n);
    return(false);
  }
  if((t=strpbrk(n,illegal_chr_in_agg_name))!=NULL)
  {
    domflip_error.set(domflip_error_t::illegal_agg_name_chr,(t-n)+1,t);
    return(false);
  }
}
//===================================================================
Aggregate::Aggregate(void)
{
  init();
}

Aggregate::Aggregate(Aggregate  & const source)
{
  init();
  copy(source);
}

Aggregate::~Aggregate(void)
{
}
//===================================================================
void Aggregate::init(void)	
{
  ListOfEntr* entrl;

  strcpy(name,"def_agg_name");
  type=1;
  entrl=new(ListOfEntr);
  agg_constr=new(ListOfConstr);
  agg_s_ent=*entrl;
}
//===================================================================
void Aggregate::clear(void)
{  
  agg_s_ent.get_list_ptr()->clear();
  agg_constr.clear();
}

//===================================================================
void Aggregate::copy(Aggregate & const source)
{ 
  strcpy(name,source.name);
  type=source.type;
  agg_evaluation=source.agg_evaluation;
  agg_constr=source.agg_constr;
  agg_s_ent.copy_entr(&source.agg_s_ent);
}
//===================================================================
Aggregate &Aggregate::operator =(Aggregate & const source)
{
  clear();
  copy(source);
  return(*this);
}
//===================================================================
bool Aggregate::operator ==(Aggregate & const source)
{
  return((bool)(type==source.type&&strcmp(name,source.name)==0));
}
//===================================================================
char *Aggregate::get_name(void)
{
  return(name);
}
//===================================================================
bool Aggregate::set_name(char *nam)
{
  if(check_agg(nam))
  {
    strcpy(name,nam);
    return(true);
  }
  else
    return(false);
}
//===================================================================
agg_type Aggregate::get_type(void)
{
  return(type);
}
//===================================================================
bool Aggregate::set_type(int i) 
{
  if (i==0)
  {
    type=pool ;
    return(true);
  };
  if (i==1)
  {
    type=plan ;
    return(true);
  };
  return(false);
}

//===================================================================
ListOfConstr Aggregate::get_agg_constr(void)
{
  return(agg_constr);
}
//===================================================================
SledForEntr *Aggregate::get_agg_s_entr(void)
{
  return( &agg_s_ent);
}
//===================================================================
bool Aggregate::write(ofstream &outfile,bool up)
{
  char		meth[LENGTH_OF_NAMES];
  int		len;


  if(!(outfile<<endl<<" "<<name<<" "<<(int)type<<endl<<"       ")) 
  {
    strcpy(meth,"Aggregate::write (name type)");
    len=strlen(meth);
    domflip_error.set(domflip_error_t::file_write_error_in,len,meth);
    return(false);
  }
  else 
  {
    if (!agg_constr.write(outfile))
      return(false);
    else
    {
      outfile<<endl<<"       ";
      if (!agg_s_ent.get_list_ptr()->write(outfile))
        return(false);
    }
  }
  return(true);
}
//=================================================================== 
bool Aggregate::write_agg(ofstream &outfile,bool up)
{
  char		meth[LENGTH_OF_NAMES];
  int		len;

  if(!(outfile<<endl<<" "<<name<<" "<<(int)type<<endl<<"       "))
  {
    strcpy(meth,"Aggregate::write_agg (name type)");
    len=strlen(meth);
    domflip_error.set(domflip_error_t::file_write_error_in,len,meth);
    return(false);
  }
}

//=================================================================== 
bool Aggregate::write_constr(ofstream &outfile,bool up)
{
  char		meth[LENGTH_OF_NAMES];
  int		len;

  if(!(outfile<<endl<<" "<<name<<" "<<(int)type<<endl<<"       "))
  {
    strcpy(meth,"Aggregate::write_constr (name type)");
    len=strlen(meth);
    domflip_error.set(domflip_error_t::file_write_error_in,len,meth);
    return(false);
  }
  else 
  {
    if (!agg_constr.write(outfile))
      return(false);
  }
  return(true);
}
//=================================================================== 
bool Aggregate::write_data(ofstream &outfile,bool up)
{
  char		meth[LENGTH_OF_NAMES];
  int		len;

  if(!(outfile<<endl<<" "<<name<<" "<<(int)type<<endl<<"       ")) 
  {
      strcpy(meth,"Aggregate::write_data (name type)");
      len=strlen(meth);
    domflip_error.set(domflip_error_t::file_write_error_in,len,meth);
    return(false);
  }
  else 
  {
    if (!agg_s_ent.get_list_ptr()->write(outfile))
      return(false);
  }
  return(true);
}

//=================================================================== 
bool Aggregate::read(ifstream &infile,bool up)
{
  int i,j;
  char na[LENGTH_OF_NAMES];
  char		meth[LENGTH_OF_NAMES];
  int		len;

  Aggregate::clear();

  if(up)
    if(!Aggregate::read(infile))
    {
      // strcpy(meth,"Aggregate::read");
      // len=strlen(meth);
      // domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
      clear();
      return(false);
    }
  
  if(!(infile>>na))
  {
    strcpy(meth,"Aggregate::read (name)");
    len=strlen(meth);
    domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
    return (false);
  }
  else
  {
    if(!check_agg(na))
    {
      strcpy(meth,"Aggregate::read (name2)");
      len=strlen(meth);
      domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
      clear();
      return (false);
    };
    set_name(na);
  }
  // cout <<"(read)Aggname: " << name << endl; // !!!!!!!!!!!

  if(!(infile>>(int &)type))
  {
    strcpy(meth,"Aggregate::read (type)");
    len=strlen(meth);
    domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
    return (false);
  };
  // cout <<"Aggtype " << type << endl;
};

//=================================================================== 
bool Aggregate::read_constr(ifstream &infile,bool up)
{
  char		meth[LENGTH_OF_NAMES];
  int		len;

  strcpy(meth,"Aggregate::read_data (number or type)");
  len=strlen(meth);
  domflip_error.set(domflip_error_t::file_read_error_in,len,meth);

  if(!agg_constr.read(infile))
    return(false);
  else 
  {
    domflip_error.set(domflip_error_t::ok);
    return(true);
  }
};

//=================================================================== 
bool Aggregate::read_data(ifstream &infile,bool up)
{
  char		meth[LENGTH_OF_NAMES];
  int		len;

  strcpy(meth,"Aggregate::read_data (number or type)");
  len=strlen(meth);
  domflip_error.set(domflip_error_t::file_read_error_in,len,meth);


  if(!agg_s_ent.get_list_ptr()->read(infile))
    return(false);
  else 
  {
  domflip_error.set(domflip_error_t::ok);
    return(true);
  }
};
//===================================================================
int Aggregate::get_nr_of_entries(void)
{
  if(get_agg_s_entr !=NULL)
    return(get_agg_s_entr()->get_how_full());
  else return(0);
}

//===================================================================
int Aggregate::get_entry(int i)
{
  return(agg_s_ent.get_entr_key(i));
}
//===================================================================
float Aggregate::get_evaluation(void)
{
  return(agg_evaluation);
}
//===================================================================
float Aggregate::evaluate_plan(void)
{
  agg_evaluation=test_eval();
  // cout<<endl<<"Eval.: "<<get_name()<<" "<<agg_evaluation<<endl;

  return(agg_evaluation);
}

//===================================================================
float Aggregate::test_eval(void)
{
  int i;
  double sum=0;
  int o1;
  int count;

  // cout<<"Aggregate::test_eval(void) "<<endl;

  count=get_nr_of_entries();
  
  for(i=1; i<count;i++)
  {
    sum = sum*10 + (get_entry(i)/10 );
  }

  for(i=1;i<count; i++)
  {
  sum = sum/10;
  }

  agg_evaluation=sum;
 	
  // cout<<"Aggregate::test_eval(void) "<<agg_evaluation<<endl;

  return(agg_evaluation);
}



