/************************************************************************
 * This library is free software; you can redistribute it and/or	*
 * modify it under the terms of the GNU Library General Public		*
 * License as published by the Free Software Foundation; either		*
 * version 2 of the License, or (at your option) any later version.	*
 * 									*
 * This library is distributed in the hope that it will be useful,	*
 * but WITHOUT ANY WARRANTY; without even the implied warranty of	*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*
 * Library General Public License for more details.			*
 * 									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the		*
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,		*
 * Boston, MA  02111-1307, USA.						*
 * 									*
 * The GNU Library General Public License file is included in		*
 * Online-Documentation							*
 *									*
 * GNU's Homepage - http://www.gnu.ai.mit.edu/home.html			*
 *									*
 * Questions & comments referring to *FLIP++: please contact us:	*
 *									*
 * Snail-Mail:								*
 *	Dr. Wolfgang SLANY						*
 *	Database and Expert System Group (E184-2)     			*
 *	Institute of Information Systems               			*
 *	Vienna University of Technology					*
 *	Paniglgasse 16, A-1040 Vienna, AUSTRIA				*
 * Telephone:								*
 *	+43-1-58801-6123     (area-code 1 maybe changed to 201 in 1998) *
 * FAX:									*
 *	+43-1-5055304	     (area-code 1 maybe changed to 201 in 1998) *
 * eMail:								*
 *	sflip@dbai.tuwien.ac.at						*
 * WWW:									*
 *	http://www.dbai.tuwien.ac.at/proj/StarFLIP/			*
 *                                                                      *
 * $Id: domagg.h,v 1.4 1997/06/24 17:28:50 reichel Exp $ *
 ***********************************************************************/


// domagg.h 

#ifndef _DOMAGG
#define _DOMAGG

#include <fstream.h>

#include "replst.h"
//===========================================================================
class PubNode1d
//===========================================================================
{
  friend class GenSled;
  friend class GenList;
public:
  void *item;
  Node1d *pred;
  Node1d *succ;
  Node1d(void) { item=NULL; }
};

enum agg_type {pool=0, plan=1};

//===========================================================================
class Aggregate 
//===========================================================================
{
protected:
  char 		name[LENGTH_OF_NAMES];
  agg_type	type;				// pool=0, plan=1
  float		agg_evaluation;
  SledForEntr	agg_s_ent;
  ListOfConstr	agg_constr;

public:
  Aggregate 	(void);
  Aggregate	(Aggregate & const);		
  ~Aggregate 	(void);
	
  void init	(void);
  void clear	(void);
  void copy	(Aggregate & const);

  Aggregate &operator =(Aggregate & const);
  bool operator ==(Aggregate & const);
  
  virtual bool write(ofstream &,bool =true);
  virtual bool write_agg(ofstream &,bool =true);
  virtual bool write_constr(ofstream &,bool =true);
  virtual bool write_data(ofstream &,bool =true);
 
  virtual bool read(ifstream &,bool =true);
  virtual bool read_constr(ifstream &,bool =true);
  virtual bool read_data(ifstream &,bool =true);
 

  int		get_nr_of_entries(void);
  int		get_entry(int);
  float		get_evaluation(void);
  float		evaluate_plan(void);

  float		test_eval(void);

  char 		*get_name(void);
  bool 		set_name (char *);

  agg_type 	get_type (void);
  bool		set_type (int); 

  ListOfConstr  get_agg_constr(void);
  SledForEntr	*get_agg_s_entr(void);

};

#endif
