/************************************************************************
 * This library is free software; you can redistribute it and/or	*
 * modify it under the terms of the GNU Library General Public		*
 * License as published by the Free Software Foundation; either		*
 * version 2 of the License, or (at your option) any later version.	*
 * 									*
 * This library is distributed in the hope that it will be useful,	*
 * but WITHOUT ANY WARRANTY; without even the implied warranty of	*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*
 * Library General Public License for more details.			*
 * 									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the		*
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,		*
 * Boston, MA  02111-1307, USA.						*
 * 									*
 * The GNU Library General Public License file is included in		*
 * Online-Documentation							*
 *									*
 * GNU's Homepage - http://www.gnu.ai.mit.edu/home.html			*
 *									*
 * Questions & comments referring to *FLIP++: please contact us:	*
 *									*
 * Snail-Mail:								*
 *	Dr. Wolfgang SLANY						*
 *	Database and Expert System Group (E184-2)     			*
 *	Institute of Information Systems               			*
 *	Vienna University of Technology					*
 *	Paniglgasse 16, A-1040 Vienna, AUSTRIA				*
 * Telephone:								*
 *	+43-1-58801-6123     (area-code 1 maybe changed to 201 in 1998) *
 * FAX:									*
 *	+43-1-5055304	     (area-code 1 maybe changed to 201 in 1998) *
 * eMail:								*
 *	sflip@dbai.tuwien.ac.at						*
 * WWW:									*
 *	http://www.dbai.tuwien.ac.at/proj/StarFLIP/			*
 *                                                                      *
 * $Id: domaglst.cpp,v 1.4 1997/06/24 17:28:51 reichel Exp $ *
 ***********************************************************************/



#include "domagglst.h"
#include "repstepsdef.h"
/*-------------------------------------------------------------------*/
/* ListOfAgg                                                         */
/*-------------------------------------------------------------------*/
// ****************************************************************** 
ListOfAgg::ListOfAgg(void) : GenList()
{}

ListOfAgg::ListOfAgg(ListOfAgg & const source) : GenList(source)
{}

ListOfAgg::ListOfAgg(SledForAgg & const source) : GenList(source)
{}

ListOfAgg::~ListOfAgg(void)
{
  clear();
}

// *********************************************************************
void ListOfAgg::del(void *ptr)
{
  if(ptr!=NULL)
    delete((Aggregate *)ptr);
}
//===================================================================
void *ListOfAgg::key(void *info)
{
  return(((Aggregate *)info)->get_name());
}
//===================================================================
bool ListOfAgg::look(void *wanted,void *info)	
{
  return((bool)(strcmp((char *)wanted,((Aggregate *)info)->get_name())==0));
}
//===================================================================
void ListOfAgg::cpy(void *&item,void *info)	
{
  item=(Aggregate *)new(Aggregate);
  (*(Aggregate *)item)=(*(Aggregate *)info);
}
//===================================================================
bool ListOfAgg::rd(ifstream &infile,void *&item)
{
  Aggregate 	temp;
  char		meth[LENGTH_OF_NAMES];
  int		len;



  item=(Aggregate *)new(Aggregate);

  if( ((Aggregate *) item)->read(infile,false))
  {
    return(true); 
  }
  else
  {
    delete ((Aggregate *)item);
    return(false);
  }
}    
//===================================================================
bool ListOfAgg::wr(ofstream &outfile,void *item)
{
  return(((Aggregate *)item)->write(outfile));
}
//===================================================================
bool ListOfAgg::wr_agg(ofstream &outfile,void *item)
{
  return(((Aggregate *)item)->write_agg(outfile));
}
//===================================================================
bool ListOfAgg::wr_constr(ofstream &outfile,void *item)
{
  return(((Aggregate *)item)->write_constr(outfile));
}
//===================================================================
bool ListOfAgg::wr_data(ofstream &outfile,void *item)
{
  return(((Aggregate *)item)->write_data(outfile));
}
//===================================================================
void ListOfAgg::clr_cons(void *item)
{
  ((Aggregate *)item)->get_agg_constr().clear();
}
//===================================================================
void ListOfAgg::clr_dat(void *item)
{
 ((Aggregate *)item)->get_agg_s_entr()->get_list_ptr()->clear();  
}
//===================================================================
bool ListOfAgg::add(Aggregate *agg,InfoCopy c)
{
  return(GenList::add((Aggregate *) agg,c));
}
//=================================================================== 
Aggregate *ListOfAgg::lookup(char *s)
{
  return((Aggregate *)get_info(GenList::lookup((void *)s)));
}
//===================================================================
bool ListOfAgg::write_agg(ofstream &outfile)
{
  SledForAgg 	help=this;
  int		i;
  char		meth[LENGTH_OF_NAMES];
  int		len;


  for(i=1;i<=no_of_items;i++)
  {
    if(wr_agg(outfile,help++))
      continue;
    else
    {
      strcpy(meth,"ListOfAgg::write_agg");
      len=strlen(meth);
      domflip_error.set(domflip_error_t::file_write_error_in,len,meth);
      return(false);
    }
  }
}
//===================================================================
bool ListOfAgg::write_constr(ofstream &outfile)
{
  SledForAgg 	help=this;
  int		i;
  char		meth[LENGTH_OF_NAMES];
  int		len;


  for(i=1;i<=no_of_items;i++)
  {
    if(wr_constr(outfile,help++))
      continue;
    else
    {
      strcpy(meth,"ListOfAgg::write_constr");
      len=strlen(meth);
      domflip_error.set(domflip_error_t::file_write_error_in,len,meth);
      return(false);
    }
  }
}
//===================================================================
bool ListOfAgg::write_data(ofstream &outfile)
{
  SledForAgg 	help=this;
  int		i;
  char		meth[LENGTH_OF_NAMES];
  int		len;


  for(i=1;i<=no_of_items;i++)
  {
    if(wr_data(outfile,help++))
      continue;
    else
    {
      strcpy(meth,"ListOfAgg::write_data");
      len=strlen(meth);
      domflip_error.set(domflip_error_t::file_write_error_in,len,meth);
      return(false);
    }
  }
}

//===================================================================
void ListOfAgg::clear_cons()
{
  SledForAgg 	help=this;
  int		i;
  char		meth[LENGTH_OF_NAMES];
  int		len;

  while(no_of_items>0)
  {
    clr_cons(help++);
  }

}

//===================================================================
void ListOfAgg::clear_dat()
{
  SledForAgg 	help=this;
  int		i;
  char		meth[LENGTH_OF_NAMES];
  int		len;

  while(no_of_items>0)
  {
    clr_dat(help++);
  }
}

//===================================================================

ListOfAgg &ListOfAgg::operator =(ListOfAgg & const source)
{
  copy(source);
  return(*this);
}

//===================================================================
ListOfAgg &ListOfAgg::operator =(SledForAgg & const source)
{
  copy(source);
  return(*this);
}
//===================================================================
/*
bool ListOfAgg::operator ==(ListOfAgg & const par)
{
  return(GenList::operator ==(par));
}
*/

//*******************************************************************
//===================================================================
ListOfAgg * SledForAgg::get_list_ptr(void)
{
  return ((ListOfAgg *)list_ptr);
}

//===================================================================
int SledForAgg::get_nr_of_entries(void)
{
  if( operator()() != NULL)
    return( ((Aggregate *)operator()())->get_nr_of_entries());
  else return(0);
}
//===================================================================
int SledForAgg::get_entry(int i)
{
  if( operator()() != NULL)
    return(((Aggregate *)operator()())->get_entry(i));
  else return(0);
}
//===================================================================
float SledForAgg::get_evaluation(void)
{
  if( operator()() != NULL)
    return(((Aggregate *)operator()())->get_evaluation());
  else return(0);
}
//===================================================================
float SledForAgg::evaluate_plan(void)	// float ????????
{
  if( operator()() != NULL)
    return(((Aggregate *)operator()())->evaluate_plan());
  else return(0);
}

//===================================================================
void SledForAgg::copy_sched(SledForAgg * source)
{
  list_ptr=new(ListOfAgg);
  *list_ptr=*((ListOfAgg *)(source->list_ptr));
  first();
  operator[](source->index);
}
/*
//===================================================================
bool SledForAgg::call_repair(int f,int pos,ind i)
{
  ind i1,i2;
  int tmp;

  // debug = 0;

  if ( operator()() != NULL)
  {
    tmp= position();

    switch(f)
    {
      case A_MOVE_FWD:					//   <-
      {
	if(i.pla < 2) return(false);
	i1.agg=i.agg;		i2.agg=i.agg;
        i1.pla=i.pla; 		i2.pla=i.pla-1;		
	i1.num=i.num;		i2.num=1;
    	if(debuginfo == 1)
	{
          cout<<"  MOVE_FWD: ";
          cout<<"     Pos1: "<<i1.agg<<" "<<i1.pla<<" "<<i1.num;
          cout<<"     Pos2: "<<i2.agg<<" "<<i2.pla<<" "<<i2.num<<"    "<<endl;
	}
        return(move(i1,i2));
      }

      case A_MOVE_BWD:					//   ->
      {
	i1.agg=i.agg;		i2.agg=i.agg;
        i1.pla=i.pla; 		i2.pla=i.pla+1;		
	i1.num=i.num;		i2.num=1;
    	if(debuginfo == 1)
	{
          cout<<"  MOVE_BWD: ";
          cout<<"     Pos1: "<<i1.agg<<" "<<i1.pla<<" "<<i1.num;
          cout<<"     Pos2: "<<i2.agg<<" "<<i2.pla<<" "<<i2.num<<"    "<<endl;
	}

        return(move(i1,i2));
      }


      case A_MOVE_GRP_FWD:				// <- 2 items, 2 pl
      {
	i1.agg=i.agg;		i2.agg=i.agg;
        i1.pla=i.pla; 		i2.pla=i.pla-2;
	i1.num=2;		i2.num=2;
    	if(debuginfo == 1)
	{
          cout<<"  MOVE_GRP_FWD: ";
          cout<<"     Pos1: "<<i1.agg<<" "<<i1.pla<<" "<<i1.num;
          cout<<"     Pos2: "<<i2.agg<<" "<<i2.pla<<" "<<i2.num<<"    "<<endl;
	}
        return(move(i1,i2));
      }

      case A_MOVE_GRP_BWD:				// -> 2 items, 2 pl
      {
	i1.agg=i.agg;		i2.agg=i.agg;
        i1.pla=i.pla; 		i2.pla=i.pla+2;
	i1.num=2;		i2.num=2;
    	if(debuginfo == 1)
	{
          cout<<"  MOVE_GRP_BWD: ";
          cout<<"     Pos1: "<<i1.agg<<" "<<i1.pla<<" "<<i1.num;
          cout<<"     Pos2: "<<i2.agg<<" "<<i2.pla<<" "<<i2.num<<"    "<<endl;
	}
        return(move(i1,i2));
      }


      case A_CHANGE:
      {
	i1.agg=i.agg;		i2.agg=i1.agg;
        i1.pla=i.pla;		i2.pla=1;
 	i1.num=1;		i2.num=1;
    	if(debuginfo == 1)
	{
          cout<<"  CHANGE: ";
          cout<<"     Pos1: "<<i1.agg<<" "<<i1.pla<<" "<<i1.num;
          cout<<"     Pos2: "<<i2.agg<<" "<<i2.pla<<" "<<i2.num<<"    "<<endl;
	}
        return(change(i1,i2));
      }
 
      case A_INSERT:					// from 1 into 2
      {
        if(i.agg = 2) return(false);
        i1.agg=1;		i2.agg=2;
        i1.pla=i.pla;		i2.pla=1;  	
	i1.num=1;		i2.num=1;
    	if(debuginfo == 1)
	{
          cout<<"  INSERT: ";
          cout<<"     Pos1: "<<i1.agg<<" "<<i1.pla<<" "<<i1.num;
          cout<<"     Pos2: "<<i2.agg<<" "<<i2.pla<<" "<<i2.num<<"    "<<endl;
	}
        return(move(i1,i2));
      }

      case A_REMOVE:					// from 2 into 1
      {
        if(i.agg = 1) return(false);
        i1.agg=2; 		i2.agg=1;
        i1.pla=i.pla;		i2.pla=1;
 	i1.num=1;		i2.num=1;
    	if(debuginfo == 1)
	{
          cout<<"  REMOVE: ";
          cout<<"     Pos1: "<<i1.agg<<" "<<i1.pla<<" "<<i1.num;
          cout<<"     Pos2: "<<i2.agg<<" "<<i2.pla<<" "<<i2.num<<"    "<<endl;
	}
        return(move(i1,i2));
      }

      case A_MOVE_TO_END:
      {
	i1.agg= i.agg; 		i2.agg= i.agg;
        i1.pla=i.pla; 		i2.pla=get_nr_of_entries(); 
	i1.num=1;		i2.num=1;
    	if(debuginfo == 1)
	{
          cout<<"  MOVE_TO_END: ";
          cout<<"     Pos1: "<<i1.agg<<" "<<i1.pla<<" "<<i1.num;
          cout<<"     Pos2: "<<i2.agg<<" "<<i2.pla<<" "<<i2.num<<"    "<<endl;
	}
        return(move(i1,i2));
      }

      case A_MOVE_TO_BEG:
      {
	i1.agg= i.agg;        	i2.agg= i.agg;
        i1.pla=i.pla;		i2.pla=1; 
 	i1.num=1;        	i2.num=1;
    	if(debuginfo == 1)
	{
          cout<<"  MOVE_TO_BEG: ";
          cout<<"     Pos1: "<<i1.agg<<" "<<i1.pla<<" "<<i1.num;
          cout<<"     Pos2: "<<i2.agg<<" "<<i2.pla<<" "<<i2.num<<"    "<<endl;
	}
        return(move(i1,i2));
      }

  
    }
  }
  else 
    cout<<"ERROR: SledForAgg::call_repair() NO AGGREGAT "<<endl;
}


*/
//===================================================================
void SledForAgg::show_sched(void)
{
  cout<<endl<<"*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*"<<endl;
  cout<<"Aggregate: "<<((Aggregate *)operator()())->get_name()<<endl;
  cout<<"Evaluation: "<<((Aggregate *)operator()())->get_evaluation()<<endl;

  ((ListOfAgg *)get_list_ptr())->show();
  cout<<endl<<"*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*"<<endl;
}




//===================================================================
bool SledForAgg::change(ind i1, ind i2)
{
  Aggregate 	*a1;
  Aggregate 	*a2;
  void 		*temp1;
  void 		*temp2;
  void 		*help;
  int		 j;
  int		count;



  if(debuginfo == 1)
  {
    cout<<"     Pos1: "<<i1.agg<<" "<<i1.pla<<" "<<i1.num;
    cout<<"     Pos2: "<<i2.agg<<" "<<i2.pla<<" "<<i2.num<<"    "<<endl;
  }


  if((i1.num!=i2.num) || (i1.num<1))
  {
    if(debuginfo == 1) cout<<"change: num not valid"<<endl;
    return(false);
  }

  if((i1.pla<1) || (i2.pla<1))
  {
    if(debuginfo == 1) cout<<"change: place is less then 1"<<endl;
    return(false);
  }

  if((i1.agg == i2.agg) && (abs(i2.pla-i1.pla)<i1.num))
  {
    if(debuginfo == 1) cout<<"change: scope overlap"<<endl;
    return(false);
  }

  a1=((Aggregate *)operator[](i1.agg));
  if(a1==NULL)
  {
    if(debuginfo == 1) cout<<"Error in change: no Agg1"<< endl;
    return(false);
  };

  a2=((Aggregate *)operator[](i2.agg));
  if(a2==NULL)
  {
    if(debuginfo == 1) cout<<"Error in change: no Agg2"<< endl;
    return(false);
  };

  count=a1->get_agg_s_entr()->get_how_full();

  if((i1.pla + i1.num - 1) > count)		// not so many el.
  {
    if(debuginfo == 1) cout<<"change: Not so many object in list 1"<<endl;
    return(false);
  }

  count=a2->get_agg_s_entr()->get_how_full();
  if((i2.pla + i2.num - 1) > count)		// not so many el.
  {
    if(debuginfo == 1) cout<<"change: Not so many object in list 2"<<endl;
    return(false);
  }

  for(j=0;j<i1.num;j++)
  {
    temp1=a1->get_agg_s_entr()->get_ptr((i1.pla)+j);
    temp2=a2->get_agg_s_entr()->get_ptr((i2.pla)+j);

    help=((PubNode1d *)temp1)->item;

    ((PubNode1d *)temp1)->item=((PubNode1d *)temp2)->item;
    ((PubNode1d *)temp2)->item=help; 
  }  
}


//===================================================================
bool SledForAgg::move(ind i1, ind i2)
{
  Aggregate 	*a1;
  Aggregate 	*a2;
  void 		*temp1;
  int		j,k;
  int 		count;


  if(debuginfo == 1)
  {
    cout<<"     Pos1: "<<i1.agg<<" "<<i1.pla<<" "<<i1.num;
    cout<<"     Pos2: "<<i2.agg<<" "<<i2.pla<<" "<<i2.num<<"    "<<endl;
  }

  if((i1.num!=i2.num) || (i1.num<1))
  {
    if(debuginfo == 1) cout<<"move: num not valid"<<endl;
    return(false);
  }

  if((i1.pla<1) || (i2.pla<1))
  {
    if(debuginfo == 1) cout<<"move: place is less then 1"<<endl;
    return(false);
  }

  a1=((Aggregate *)operator[](i1.agg));
  if(a1==NULL)
  {
    if(debuginfo == 1) cout<<"Error in move: no Agg1"<< endl;
    return(false);
  };

  count=a1->get_agg_s_entr()->get_how_full();
  if((i1.pla + i1.num - 1) > count)		// not so many el.
  {
    if(debuginfo == 1) cout<<"move: Not so many objects in list 1"<<endl;
    return(false);
  }

  if(i1.agg==i2.agg)				// *** equal Aggr
  {
    // cout<<"agg =="<<endl;
    if(i1.pla==i2.pla) 
    {
      // cout<<"agg pl ="<<endl;
      return(true);
    }

    if(i1.pla<i2.pla)				// forward
    {
      //cout<<"agg pl<"<<endl;

      if((i2.pla + i1.num - 1) > count)		// not so many el.
      {
        if(debuginfo == 1) cout<<"move: num to big for new place"<<endl;
        return(false);
      }


      for(j=0;j<i1.num;j++)
      {
        temp1=a1->get_agg_s_entr()->get_ptr((i1.pla)+j);
        a1->get_agg_s_entr()->get_ptr((i2.pla)+(i1.num)+j-1);
        //cout<<"Index1: "<<a1->get_agg_s_entr()->position()<<endl;
        a1->get_agg_s_entr()->insert(((PubNode1d *)temp1)->item,copy_when_insert);
      }

      for(k=0;k<i1.num;k++)
      {
        a1->get_agg_s_entr()->operator[]((i1.pla));
        //cout<<"Index2: "<<a1->get_agg_s_entr()->position()<<endl;
        a1->get_agg_s_entr()->kick_out();
      }
    }
    else					// backward
    {
      //cout<<"agg pl>"<<endl;
      for(j=0;j<i1.num;j++)
      { 
        temp1=a1->get_agg_s_entr()->get_ptr((i1.pla)+j+j);
        a1->get_agg_s_entr()->get_ptr((i2.pla)+j-1);
        //cout<<"Index1: "<<a1->get_agg_s_entr()->position()<<endl;
        a1->get_agg_s_entr()->insert(((PubNode1d *)temp1)->item,copy_when_insert);
      }
      for(k=0;k<i1.num;k++)
      {
        a1->get_agg_s_entr()->operator[]((i1.pla)+i1.num);
        //cout<<"Index2: "<<a1->get_agg_s_entr()->position()<<endl;
        a1->get_agg_s_entr()->kick_out();
      }
    }
  }
  else						// *** different Aggs
  {
    //cout<<"dif agg "<<endl;

    a2=((Aggregate *)operator[](i2.agg));
    if(a2==NULL)
    {
      if(debuginfo == 1) cout<<"Error in move: no Agg2"<< endl;
      return(false);
    };

    count=a2->get_agg_s_entr()->get_how_full();
    if(i2.pla - 1  > count)		// not so many el.
    {
      if(debuginfo == 1) cout<<"move: Not so many objects in list 2"<<endl;
      return(false);
    }

    for(j=0;j<i1.num;j++)
    {
      temp1=a1->get_agg_s_entr()->get_ptr((i1.pla)+j);
      a2->get_agg_s_entr()->get_ptr((i2.pla)+j-1);
      // cout<<"Index1: "<<a2->get_agg_s_entr()->position()<<endl;
      a2->get_agg_s_entr()->insert(((PubNode1d *)temp1)->item,copy_when_insert);
    }
    for(k=0;k<i1.num;k++)
    {
      a1->get_agg_s_entr()->operator[]((i1.pla));
      //cout<<"Index2: "<<a1->get_agg_s_entr()->position()<<endl;
      a1->get_agg_s_entr()->kick_out();
    }
  }
} 

 
