/************************************************************************
 * This library is free software; you can redistribute it and/or	*
 * modify it under the terms of the GNU Library General Public		*
 * License as published by the Free Software Foundation; either		*
 * version 2 of the License, or (at your option) any later version.	*
 * 									*
 * This library is distributed in the hope that it will be useful,	*
 * but WITHOUT ANY WARRANTY; without even the implied warranty of	*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*
 * Library General Public License for more details.			*
 * 									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the		*
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,		*
 * Boston, MA  02111-1307, USA.						*
 * 									*
 * The GNU Library General Public License file is included in		*
 * Online-Documentation							*
 *									*
 * GNU's Homepage - http://www.gnu.ai.mit.edu/home.html			*
 *									*
 * Questions & comments referring to *FLIP++: please contact us:	*
 *									*
 * Snail-Mail:								*
 *	Dr. Wolfgang SLANY						*
 *	Database and Expert System Group (E184-2)     			*
 *	Institute of Information Systems               			*
 *	Vienna University of Technology					*
 *	Paniglgasse 16, A-1040 Vienna, AUSTRIA				*
 * Telephone:								*
 *	+43-1-58801-6123     (area-code 1 maybe changed to 201 in 1998) *
 * FAX:									*
 *	+43-1-5055304	     (area-code 1 maybe changed to 201 in 1998) *
 * eMail:								*
 *	sflip@dbai.tuwien.ac.at						*
 * WWW:									*
 *	http://www.dbai.tuwien.ac.at/proj/StarFLIP/			*
 *                                                                      *
 * $Id: domlib.cpp,v 1.4 1997/06/24 17:28:58 reichel Exp $ *
 ***********************************************************************/


// domlib.cpp

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fstream.h>
#include <time.h>


#include "domlib.h"


/*-------------------------------------------------------------------*/
/*  Domain                                                  	     */
/*-------------------------------------------------------------------*/
// ******************************************************************
Domain::Domain(void)
{
  init();
}
//===================================================================
Domain::~Domain(void)
{}
//===================================================================
void Domain::init(void)		
{
  ListOfAgg* aggl1;

  strcpy(name,"");
  evaluation=0;
  optmet=0;					// default: TabuAlg
  adv_idx.agg=0; adv_idx.pla=0; adv_idx.num=0;
  rep_step=0;
  aggl1=new(ListOfAgg);
  aggs=*aggl1;
  dom_constr=new(ListOfConstr);
  repairl=new(ListOfRepairs);
  c_violl=new(ListOfCviol);
}
//===================================================================
void Domain::copy(Domain & const source)
{
  strcpy(name,source.name);
  evaluation=source.evaluation;
  optmet=source.optmet;
  adv_idx=source.adv_idx;
  rep_step=source.rep_step;
  aggs.copy_sched(&source.aggs);			// aggs=aggl;
  dom_constr=source.dom_constr;
  repairl=source.repairl;
  c_violl=source.c_violl;
}
//===================================================================
void Domain::open(void)		// for test only : opens all files
{
  open_dom_constr("dom2.dcon");
  //cout<<"***Domain::init:open_dom_constr"<<endl;
  //dom_constr.show();
  open_aggregats("dom2.aggs");
  //cout<<"***Domain::init:open_aggregats"<<endl;
  open_agg_constr("dom2.acon");
  //cout<<"***Domain::init:open_agg_constr"<<endl;
  //show_aggregats();
  open_data("dom2.data");
  //cout<<"***Domain::init:open_data"<<endl;
  //show_aggregats();
  open_c_violl("dom2.cvio");
  //c_violl.show();
  //cout<<"***Domain::init:open_c_violl"<<endl;
  open_repairl("dom2.reps");
  //cout<<"***Domain::init:open_c_violl"<<endl;
  //repairl.show();
}

//===================================================================
void Domain::export(char *name)
{
  ofstream outfile;

  outfile.open(name,ios::out|ios::trunc);
  if(outfile.is_open())
  {
    if(!aggs.get_list_ptr()->write(outfile))
      cout<<domflip_error.get_txt_eval()<<endl;
    outfile.close();
  }
}
// ******************************************************************
void Domain::open_aggregats(char *name)
{
  ifstream infile;

  infile.open(name,ios::in);
  if(infile.is_open())
  {
    if(!aggs.get_list_ptr()->read(infile))
    {
      cout << domflip_error.get_txt_eval() << endl;
      exit(-1);
    }
  
  };
  infile.close();
}
//===================================================================
void Domain::export_aggregats(char *name)
{
  ofstream 	outfile;
  int		sum;

  sum=aggs.get_how_full();

  outfile.open(name,ios::out|ios::trunc);
  if(outfile.is_open())
  {
    outfile<<sum<<" "<<aggs.get_list_ptr()->get_flags();
    if(!aggs.get_list_ptr()->write_agg(outfile))
    {
      cout<<domflip_error.get_txt_eval()<<endl;
      exit(-1);
    }
  }
  outfile.close();
}

//===================================================================
void Domain::clear_aggregats(void)
{
  aggs.get_list_ptr()->clear();
}

//===================================================================
void Domain::show_aggregats(void)
{
  aggs.get_list_ptr()->show();
}
// ******************************************************************
void Domain::open_dom_constr(char *name)
{
  ifstream 	infile;
  char		meth[LENGTH_OF_NAMES];
  int		len;


  strcpy(meth,"Domain::open_dom_constr ");
  len=strlen(meth);
  domflip_error.set(domflip_error_t::file_read_error_in,len,meth);

  infile.open(name,ios::in);
  if(infile.is_open())
  {
    if(!dom_constr.read(infile))
    {
      cout<<domflip_error.get_txt_eval()<<endl;
      exit(-1);
    } 
  }
  infile.close();
  domflip_error.set(domflip_error_t::ok);

}
// ******************************************************************
void Domain::export_dom_constr(char *name)
{
  ofstream 	outfile;
  char		meth[LENGTH_OF_NAMES];
  int		len;


  strcpy(meth,"Domain::open_dom_constr ");
  len=strlen(meth);
  domflip_error.set(domflip_error_t::file_read_error_in,len,meth);


  outfile.open(name,ios::out|ios::trunc);
  if(outfile.is_open())
  {
    if(!dom_constr.write(outfile))
      cout<<domflip_error.get_txt_eval()<<endl;
  }
  outfile.close();
  domflip_error.set(domflip_error_t::ok);
}

// ******************************************************************
void Domain::clear_dom_constr(void)
{
  dom_constr.clear();
}
// ******************************************************************
void Domain::open_agg_constr(char *name)
{
  ifstream 	infile;
  char		aggname[LENGTH_OF_NAMES];
  int		co,i;
  int		x;
  char		meth[LENGTH_OF_NAMES];
  int		len;



  infile.open(name,ios::in);
  if(infile.is_open())
  {
    if(!(infile>>i))
    {
      strcpy(meth,"Domain::open_agg_constr (number of aggs)");
      len=strlen(meth);
      domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
      cout<<domflip_error.get_txt_eval()<<endl;
      exit(-1);
    };

    for(co=0; co<i; co++)
    {
      if(!(infile>>aggname))
      {
        strcpy(meth,"Domain::open_agg_constr (aggname)");
        len=strlen(meth);
        domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
        cout<<domflip_error.get_txt_eval()<<endl;
        exit(-1);
      };

 

      if(!(infile>>x))				// Agg type
      {
        strcpy(meth,"Domain::open_agg_constr (aggtype)");
        len=strlen(meth);
        domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
        cout<<domflip_error.get_txt_eval()<<endl;
        exit(-1);
      };

      if(aggs.get_list_ptr()->lookup(aggname) == NULL)
      {
        strcpy(meth,"Domain::open_agg_constr (No agg with this name)");
        len=strlen(meth);
        domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
        cout << domflip_error.get_txt_eval() << endl;
        exit(-1);
      };

      if(!aggs.get_list_ptr()->lookup(aggname)->read_constr(infile))
      {
        cout << domflip_error.get_txt_eval() << endl;
        exit(-1);
      };
    };
  };
  infile.close();
}

//===================================================================
void Domain::export_agg_constr(char *name)
{
  ofstream 	outfile;
  int		sum;

  sum=aggs.get_how_full();

  outfile.open(name,ios::out|ios::trunc);
  if(outfile.is_open())
  {
    outfile<<sum<<endl;
    if(!aggs.get_list_ptr()->write_constr(outfile))
      cout<<domflip_error.get_txt_eval()<<endl;
      exit(-1);
  }
  outfile.close();
}
//===================================================================
void Domain::clear_agg_constr(void)
{
  aggs.get_list_ptr()->clear_cons();
}

// ******************************************************************
void Domain::open_data(char *name)
{
  ifstream 	infile;
  char		aggname[LENGTH_OF_NAMES];
  int		co,i;
  int		x;
  char		meth[LENGTH_OF_NAMES];
  int		len;

  infile.open(name,ios::in);
  if(infile.is_open())
  {
    if(!(infile>>i))
    {
      strcpy(meth,"Domain::open_data  (number of aggs)");
      len=strlen(meth);
      domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
      cout<<domflip_error.get_txt_eval()<<endl;
      exit(-1);
    };
    for(co=0; co<i; co++)
    {
      if(!(infile>>aggname))
      {
        strcpy(meth,"Domain::open_data (aggname)");
        len=strlen(meth);
        domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
        cout<<domflip_error.get_txt_eval()<<endl;
        exit(-1);
      };

      if(!(infile>>x))			// Agg type
      {
        strcpy(meth,"Domain::open_data (type)");
        len=strlen(meth);
        domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
        cout<<domflip_error.get_txt_eval()<<endl;
        exit(-1);
      };
      if(aggs.get_list_ptr()->lookup(aggname) == NULL)
      {
        strcpy(meth,"Domain::open_data (No agg with this name)");
        len=strlen(meth);
        domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
        cout << domflip_error.get_txt_eval() << endl;
        exit(-1);
      };
      if(!aggs.get_list_ptr()->lookup(aggname)->read_data(infile))
      {
        cout << domflip_error.get_txt_eval() << endl;
        exit(-1);
      }
    };
  };
  infile.close();
}

//===================================================================
void Domain::export_data(char *name)
{
  char		meth[LENGTH_OF_NAMES];
  int		len;
  ofstream	outfile;
  int		sum;

  sum=aggs.get_how_full();

  outfile.open(name,ios::out|ios::trunc);
  if(outfile.is_open())
  {
    if(!(outfile<<sum<<endl))
    {
      strcpy(meth,"Domain::export_data (sum)");
      len=strlen(meth);
      domflip_error.set(domflip_error_t::file_write_error_in,len,meth);
    };
    if(!aggs.get_list_ptr()->write_data(outfile))
    {
      cout<<domflip_error.get_txt_eval()<<endl;
      exit(-1);
    }
  }
  outfile.close();
}

//===================================================================
void Domain::clear_data()
{
  aggs.get_list_ptr()->clear_dat();
}


// ******************************************************************
void Domain::open_c_violl(char *name)
{
  ifstream 	infile;
  ofstream	outfile;
  int		sum;
  char		meth[LENGTH_OF_NAMES];
  int		len;

  strcpy(meth,"Domain::open_c_violl (number or type)");
  len=strlen(meth);
  domflip_error.set(domflip_error_t::file_read_error_in,len,meth);

  infile.open(name,ios::in);
  if(infile.is_open())
  {
    if(!c_violl.read(infile))
    {
      cout<<domflip_error.get_txt_eval()<<endl;
      exit(-1);
    }
    else
      domflip_error.set(domflip_error_t::ok);
  }
  infile.close();
}

// ******************************************************************
void Domain::open_repairl(char *name)
{
  ifstream 	infile;
  ofstream	outfile;
  int		sum;
  char		meth[LENGTH_OF_NAMES];
  int		len;

  strcpy(meth,"Domain::open_repairl (number or type)");
  len=strlen(meth);
  domflip_error.set(domflip_error_t::file_read_error_in,len,meth);

  infile.open(name,ios::in);
  if(infile.is_open())
  {
    if(!repairl.read(infile))
    {
      cout<<domflip_error.get_txt_eval()<<endl;
      exit(-1);
    }
    else
      domflip_error.set(domflip_error_t::ok);
  }
  infile.close();
}


// ******************************************************************
// ******************************************************************
void Domain::set_opt_meth(int i)
{
  if((optmet=i)>1)
  {
    domflip_error.set(domflip_error_t::set_opt_meth);
    cout<<domflip_error.get_txt_eval()<<endl;
  }
}

// ******************************************************************
int Domain::get_opt_meth()
{
  return((int)optmet);
}

// ******************************************************************
ind Domain::dom_lookup(void* name)
{
  ind		found;
  int		i,j;

  found.agg=0; found.pla=0; found.num=1;

  if(aggs.position() != 0)
  {
    ((Aggregate *)aggs.operator()())->get_agg_s_entr()->operator[](1);
    if( ((Aggregate *)aggs.operator()())->get_agg_s_entr()->lookup(name) != NULL)
    {
      found.pla = ((Aggregate *)aggs.operator()())->get_agg_s_entr()->position();
      found.agg = aggs.position();
      return(found);
    }
  }
  else
  {
    j = aggs.get_list_ptr()->get_how_full();

    for(i=1; i<=j; i++)
    {
      ((Aggregate *)aggs.operator[](i))->get_agg_s_entr()->operator[](1);
      if( ((Aggregate *)aggs.operator[](i))->get_agg_s_entr()->lookup(name) != NULL)
      {
        found.pla = ((Aggregate *)aggs.operator()())->get_agg_s_entr()->position();
        found.agg = aggs.position();
        return(found);
      }
    }
  }
  return(found);
}


SledForAgg*  Domain::get_aggs()
{
  return(&aggs);
}


// ******************************************************************
float Domain::evaluate(char * aggname)	// for test only ???
{

  return(aggs.get_list_ptr()->lookup(aggname)->test_eval());
}

// ******************************************************************
// ******************************************************************
//===================================================================
int Domain::get_nr_of_entries(void)
{
  return(aggs.get_nr_of_entries());
}
//===================================================================
int Domain::get_entry(int i)
{
  return(aggs.get_entry(i));
}
//===================================================================
float Domain::get_evaluation(void)
{
  return evaluation;
}
//===================================================================
float Domain::evaluate_plan(void)	// float ????????
{
  evaluation = aggs.evaluate_plan();
  return evaluation;
}

//===================================================================
void Domain::copy_sched(SledForAgg * source)
{
  aggs.copy_sched(source);
}
//===================================================================
bool Domain::get_advice(int i, int h)
{
  SledForCviol 		sfcv;
  ConstrViolation*	cv;
  Repair*		r;
  SledForInt		sfi;

  sfcv = c_violl;
  // h : which constraint from constraints violation list
  if ( h > sfcv.get_how_full())
  {
    // cout<<"zu weit "<<h<<" "<<sfcv.get_how_full()<< endl;
    return(false);
  }

  cv = ((ConstrViolation*)sfcv.operator[](h));
  adv_idx = cv->get_place();

  r = repairl.lookup(cv->get_name());
  sfi=*(r->get_rep_steps());

  // i: which repair step for this constraintviolation
  if(i > sfi.get_how_full())
  {
    return(false);
  }
  sfi.operator[](i);

  rep_step = (int)(sfi.get_key());

  return(true);
}
//===================================================================
void Domain::random_it(void)
{
  SledForRepairs 	sfr;
  int 	rep_max;
  int 	i, tmp;
  ind	rnd_adv;

  tmp=aggs.position();

  sfr = repairl;
  rep_max = sfr.get_how_full();
  for(i=1; i<rep_max;i++)
  {
    rnd_adv.agg=tmp;
    rnd_adv.pla=rand() % ((Aggregate *)aggs.operator[](tmp))->get_nr_of_entries();
    rnd_adv.num=1;

    adv_idx=rnd_adv;
    rep_step=1;
  }


}
//===================================================================
int Domain::pos_aggs(void)
{
  return(aggs.position());
}
//===================================================================
ListOfRepairs* Domain::get_repairl()
{
  return(&repairl);
}
//===================================================================
ListOfCviol* Domain::get_c_violl()
{
  return(&c_violl);
}

//===================================================================
bool Domain::call_repair(int pos)
{
  return(aggs.call_repair(rep_step,pos,adv_idx));
}

// ******************************************************************
// ******************************************************************
bool Domain::change(ind i1, ind i2)
{
  aggs.change(i1,i2);
}

// ******************************************************************
bool Domain::move(ind i1, ind i2)
{
  aggs.move(i1,i2);
}
// ******************************************************************
void Domain::set_evaluation(SledForAgg * aggsld)
{					// evaluate all plans
  int	i, agg_count, plan_count;
  float ev;

  aggsld->show_sched();

  agg_count=aggsld->get_how_full();
  for(i=1; i<=agg_count; i++)
  {
    ev += ((Aggregate *)aggsld->operator[](i))->get_evaluation();
    if(((Aggregate *)aggsld->operator[](i))->get_type()==1)
      plan_count++;
  }

  if(plan_count > 0) ev = ev / plan_count;

  // cout<<"***ListEval: "<<ev<<endl;

}

// Liefert die Gesamtzahl der Eintraege
// Autor: Karl Schott, 01.05.95

int Domain::get_total_number_of_entries(void)
{
   int merker = 0;

   for(int i = 1; i <= aggs.get_how_full(); i++)
       merker += ((Aggregate *)aggs.operator[](i))->get_agg_s_entr()->get_how_full();

   return merker;

}

// Liefere die x-te Position im ind-Format
// Autor: Karl Schott, 01.05.95

ind Domain::get_domain_position(int position)
{
   int merker = 0;
   int position_merker = position;
   int zw;
   ind domain_pos = {1,1,1};

   if (aggs.position() != 1)
	aggs.operator[](1);

   do {
   	zw = ((Aggregate *)aggs.operator[](domain_pos.agg))->get_agg_s_entr()->get_how_full();
	domain_pos.agg++;
	merker += zw;
	if (merker < position)
		position_merker -= zw;
   }    while (merker < position);
   domain_pos.agg--;
   domain_pos.pla = position_merker;
   return domain_pos;
}

// Suche nach den x-ten Eintrag (position) und liefere
// dessen Schluessel
// Autor: Schott Karl, 01.05.95

int Domain::get_total_entry(int position)
{
    ind pos = get_domain_position(position);

    aggs.operator[](pos.agg);

    return get_entry(pos.pla);
}

// Suche nach einem Schluessel in allen Aggregaten
// Autor: Schott Karl, 01.05.95

ind Domain::dom_total_lookup(void *to_search_for)
{
    ind position;
	
    for(int i=1; i<= aggs.get_how_full(); i++)
    {
	aggs.operator[](i);
	position = dom_lookup(to_search_for);
	if (position.agg != 0)
		return position;
    }
}
