/************************************************************************
 * This library is free software; you can redistribute it and/or	*
 * modify it under the terms of the GNU Library General Public		*
 * License as published by the Free Software Foundation; either		*
 * version 2 of the License, or (at your option) any later version.	*
 * 									*
 * This library is distributed in the hope that it will be useful,	*
 * but WITHOUT ANY WARRANTY; without even the implied warranty of	*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*
 * Library General Public License for more details.			*
 * 									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the		*
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,		*
 * Boston, MA  02111-1307, USA.						*
 * 									*
 * The GNU Library General Public License file is included in		*
 * Online-Documentation							*
 *									*
 * GNU's Homepage - http://www.gnu.ai.mit.edu/home.html			*
 *									*
 * Questions & comments referring to *FLIP++: please contact us:	*
 *									*
 * Snail-Mail:								*
 *	Dr. Wolfgang SLANY						*
 *	Database and Expert System Group (E184-2)     			*
 *	Institute of Information Systems               			*
 *	Vienna University of Technology					*
 *	Paniglgasse 16, A-1040 Vienna, AUSTRIA				*
 * Telephone:								*
 *	+43-1-58801-6123     (area-code 1 maybe changed to 201 in 1998) *
 * FAX:									*
 *	+43-1-5055304	     (area-code 1 maybe changed to 201 in 1998) *
 * eMail:								*
 *	sflip@dbai.tuwien.ac.at						*
 * WWW:									*
 *	http://www.dbai.tuwien.ac.at/proj/StarFLIP/			*
 *                                                                      *
 * $Id: domlib.h,v 1.5 1997/06/24 17:28:59 reichel Exp $ *
 ***********************************************************************/


// domlib.h

#ifndef _DOMLIB
#define _DOMLIB

#include "domaglst.h"

enum optmethode {TabuAlg=0,GeneticAlg=1};
// optinitpar


//===========================================================================
class Domain 
//===========================================================================
{
protected:
  char 		name[LENGTH_OF_NAMES];
  float		evaluation;	// evaluation value
  optmethode	optmet;		// selected optimize method
  // optinitpar	optpar;		// initialize optimize param
  ind		adv_idx;	// index for next repair
  int		rep_step;	// next repair step
  SledForAgg	aggs;		// sled for aggregats in domain
  ListOfConstr	dom_constr;	// domain constraints
  ListOfCviol	c_violl;	// list with all constraints
  ListOfRepairs repairl;	// constr violation -> repair step 
  


public:
  Domain(void);
  ~Domain(void);

  void		init(void);	

  void		copy(Domain & const source);
  void		open(void);		// gets default files
  void		export(char *);		// 

  void		open_aggregats(char *);	// read aggregats from file
  void		export_aggregats(char *); // writes aggregats,cons,data
  void		clear_aggregats(void);	// clears data,cons and aggs
  void		clear_agg(char *);
  void		show_aggregats(void);	// display aggs, con and data

  void		open_dom_constr(char *);	
  void		export_dom_constr(char *);
  void		clear_dom_constr(void);	

  void		open_agg_constr(char *);	
  void		export_agg_constr(char *);	
  void		clear_agg_constr(void);

  void		open_data(char *);	// read sched. objects from file
  void		export_data(char *);	// wr. all obj. for all aggs into file
  void		clear_data(void);	// clears objects for all aggs

  void		open_c_violl(char *);	

  void		open_repairl(char *);	
  void		export_repairl(char *);
  void		clear_repairl(void);


  void		set_opt_meth( int);   
  int		get_opt_meth(void);	// return optimize method 
  ind		dom_lookup(void *);

  SledForAgg*	get_aggs(void);

  float 	evaluate(char *);


 
  int		get_nr_of_entries(void);
  int		get_entry(int);
  float		get_evaluation(void);
  ListOfAgg *	get_list_ptr(void);
  float		evaluate_plan(void);
  void 		copy_sched(SledForAgg *);
  bool		get_advice(int, int);
  void		random_it(void);
  int 		pos_aggs(void);

  ListOfRepairs* get_repairl(void);
  ListOfCviol*	 get_c_violl(void);

  bool  	call_repair(int);


  bool 		change(ind, ind);	// repair step
  bool		move(ind, ind);		// repair step

  void 		set_evaluation(SledForAgg * );
  int           get_total_number_of_entries(void);
  ind           get_domain_position(int position);
  int           get_total_entry(int position);
  ind           dom_total_lookup(void *to_search_for);


};


#endif
