/************************************************************************
 * This library is free software; you can redistribute it and/or	*
 * modify it under the terms of the GNU Library General Public		*
 * License as published by the Free Software Foundation; either		*
 * version 2 of the License, or (at your option) any later version.	*
 * 									*
 * This library is distributed in the hope that it will be useful,	*
 * but WITHOUT ANY WARRANTY; without even the implied warranty of	*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*
 * Library General Public License for more details.			*
 * 									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the		*
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,		*
 * Boston, MA  02111-1307, USA.						*
 * 									*
 * The GNU Library General Public License file is included in		*
 * Online-Documentation							*
 *									*
 * GNU's Homepage - http://www.gnu.ai.mit.edu/home.html			*
 *									*
 * Questions & comments referring to *FLIP++: please contact us:	*
 *									*
 * Snail-Mail:								*
 *	Dr. Wolfgang SLANY						*
 *	Database and Expert System Group (E184-2)     			*
 *	Institute of Information Systems               			*
 *	Vienna University of Technology					*
 *	Paniglgasse 16, A-1040 Vienna, AUSTRIA				*
 * Telephone:								*
 *	+43-1-58801-6123     (area-code 1 maybe changed to 201 in 1998) *
 * FAX:									*
 *	+43-1-5055304	     (area-code 1 maybe changed to 201 in 1998) *
 * eMail:								*
 *	sflip@dbai.tuwien.ac.at						*
 * WWW:									*
 *	http://www.dbai.tuwien.ac.at/proj/StarFLIP/			*
 *                                                                      *
 * $Id: domlib1.cpp,v 1.4 1997/06/24 17:29:00 reichel Exp $ *
 ***********************************************************************/


// domlib.cpp

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fstream.h>

#include "domlib.h"

/*-------------------------------------------------------------------*/
/* RepSteps                                                  	     */
/*-------------------------------------------------------------------*/
// ******************************************************************
RepSteps::RepSteps(void) {};
RepSteps::~RepSteps(void) {};

void RepSteps::init(void) 
{
  int c,r; 

  for(c=0;c<MAX_NO_OF_CONSTR-1; c++) 
  { 
    for(r=0;r<MAX_NO_OF_REP_STEPS-1; r++)
    {
      rule[c][r]=r;
    };
  };
};

/*-------------------------------------------------------------------*/
/*  Domain                                                  	     */
/*-------------------------------------------------------------------*/
// ******************************************************************
Domain::Domain(void)
{
  aggs=aggl;
}
//===================================================================
Domain::~Domain(void)
{
}

//===================================================================
void Domain::init(void)		// for test only : opens all files
{
  open_aggregats("dom1e.aggs");
  cout<<"***Domain::init:open_aggregats"<<endl;
  open_constr("dom1e.cons");
  cout<<"***Domain::init:open_constr"<<endl;
  open_data("dom1e.data");
  cout<<"***Domain::init:open_data"<<endl;

  // import_repstep();
  // import_optmet();
  // import_evrules();

}
//===================================================================
bool Domain::export(char *name)
{
  ofstream outfile;

  outfile.open(name,ios::out|ios::trunc);
  if(outfile.is_open())
  {
    if(!aggl.write(outfile))
      cout<<domflip_error.get_txt_eval()<<endl;
    outfile.close();
  }
}

// ******************************************************************
void Domain::open_aggregats(char *name)
{
  ifstream infile;

  infile.open(name,ios::in);
  if(infile.is_open())
  {
    if(!aggl.read(infile))
    {
      cout << domflip_error.get_txt_eval() << endl;
      cout << "Fehler aggdom1.read(infile) " << endl; 	// ++
    };
  };
  infile.close();
}
//===================================================================
bool Domain::export_aggregats(char *name)
{
  ofstream 	outfile;
  int		sum;

  sum=aggs.get_how_full();
  //cout<<"SUM: "<<sum<<endl;

  outfile.open(name,ios::out|ios::trunc);
  if(outfile.is_open())
  {
    outfile<<sum<<" "<<aggl.get_flags();
    if(!aggl.write_agg(outfile))
    {
      cout<<domflip_error.get_txt_eval()<<endl;
      outfile.close();
      return(false);
    }
  }
  outfile.close();
}

//===================================================================
void Domain::clear_aggregats(void)
{
  while(aggs.get_how_full()>0)
  {
    aggs.first();
    aggs.kick_out();
    //cout << "Anz. " << aggs.get_how_full() << endl;
  }
}

//===================================================================
void Domain::show_aggregats(void)
{
  aggl.show();
}

// ******************************************************************
void Domain::open_constr(char *name)
{
  ifstream 	infile;
  char		aggname[LENGTH_OF_NAMES];
  int		co,i;
  int		x;


  infile.open(name,ios::in);
  if(infile.is_open())
  {
    if(!(infile>>i))
    {
      domflip_error.set(domflip_error_t::file_read_error);
      infile.close();
      return (false);
    };
    // cout <<"CONSTR COUNT: "<< i<< endl;

    for(co=0; co<i; co++)
    {
      if(!(infile>>aggname))
      {
        domflip_error.set(domflip_error_t::file_read_error);
        infile.close();
        return (false);
      };
      infile>>x;				// Agg type
      if(!aggl.lookup(aggname)->read_constr(infile))
      {
        cout << domflip_error.get_txt_eval() << endl;
        cout << "Fehler aggdom1.read(infile) " << endl; 	// ++
      };
    };
  };
  infile.close();
}

//===================================================================
bool Domain::export_constr(char *name)
{
  ofstream 	outfile;
  int		sum;

  sum=aggs.get_how_full();
  //cout<<"SUM: "<<sum<<endl;

  outfile.open(name,ios::out|ios::trunc);
  if(outfile.is_open())
  {
    outfile<<sum<<endl;
    if(!aggl.write_constr(outfile))
    {
      cout<<domflip_error.get_txt_eval()<<endl;
      outfile.close();
      return(false);
    }
  }
  outfile.close();
}


//===================================================================
void Domain::clear_constr(void)
{
  int i, sum;

  sum=aggs.get_how_full();

  for(i=0;i<count-1;i++)
  {
    aggl.clear_cons();
    // !!!!!!!!!!!!!!!!!!!!!!!***************!!!!!!!!!!!!!!!!!
    //aggs.kick_out();
    //cout << "Anz. " << aggs.get_how_full() << endl;
  }
}

// ******************************************************************
void Domain::open_data(char *name)
{
  ifstream 	infile;
  char		aggname[LENGTH_OF_NAMES];
  int		co,i;
  int		x;

  infile.open(name,ios::in);
  if(infile.is_open())
  {
    if(!(infile>>i))
    {
      domflip_error.set(domflip_error_t::file_read_error);
      return (false);
    };
    for(co=0; co<i; co++)
    {
      if(!(infile>>aggname))
      {
        domflip_error.set(domflip_error_t::file_read_error);
        return (false);
      };
      infile>>x;			// Agg type
      if(!aggl.lookup(aggname)->read_data(infile))
      {
        cout << domflip_error.get_txt_eval() << endl;
        cout << "Fehler aggdom1.read(infile) " << endl; 	// ++
      };
    };
  };
  infile.close();
}

//===================================================================
bool Domain::export_data(char *name)
{
  ofstream	outfile;
  int		sum;

  sum=aggs.get_how_full();
  //cout<<"SUM: "<<sum<<endl;

  outfile.open(name,ios::out|ios::trunc);
  if(outfile.is_open())
  {
    outfile<<sum<<endl;
    if(!aggl.write_data(outfile))
    {
      cout<<domflip_error.get_txt_eval()<<endl;
      outfile.close();
      return(false);
    }
  }
  outfile.close();
}
/*
//===================================================================
void Domain::clear_data()
{
}
// ******************************************************************
bool Domain::open_repstep(char *name)
{
}
//===================================================================
bool Domain::export_repstep(char *name)
{
}
//===================================================================
void Domain::clear_repstep()
{
}

// ******************************************************************
bool Domain::open_optmet(char *name)
{
}
//===================================================================
bool Domain::export_optmet(char *name)
{
}
//===================================================================
void Domain::clear_optmet()
{
}

// ******************************************************************
bool Domain::open_evrules(char *name)
{
}
//===================================================================
bool Domain::export_evrules(char *name)
{
}
//===================================================================
void Domain::clear_evrules()
{
}
*/


int Domain::get_opt_meth()
{
  return((int)optmet);
}


// ******************************************************************
bool Domain::change(ind i1, ind i2)
{
  aggs.change(i1,i2);
}


// ******************************************************************
bool Domain::move(ind i1, ind i2)
{
  aggs.move(i1,i2);
}





