/************************************************************************
 * This library is free software; you can redistribute it and/or	*
 * modify it under the terms of the GNU Library General Public		*
 * License as published by the Free Software Foundation; either		*
 * version 2 of the License, or (at your option) any later version.	*
 * 									*
 * This library is distributed in the hope that it will be useful,	*
 * but WITHOUT ANY WARRANTY; without even the implied warranty of	*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*
 * Library General Public License for more details.			*
 * 									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the		*
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,		*
 * Boston, MA  02111-1307, USA.						*
 * 									*
 * The GNU Library General Public License file is included in		*
 * Online-Documentation							*
 *									*
 * GNU's Homepage - http://www.gnu.ai.mit.edu/home.html			*
 *									*
 * Questions & comments referring to *FLIP++: please contact us:	*
 *									*
 * Snail-Mail:								*
 *	Dr. Wolfgang SLANY						*
 *	Database and Expert System Group (E184-2)     			*
 *	Institute of Information Systems               			*
 *	Vienna University of Technology					*
 *	Paniglgasse 16, A-1040 Vienna, AUSTRIA				*
 * Telephone:								*
 *	+43-1-58801-6123     (area-code 1 maybe changed to 201 in 1998) *
 * FAX:									*
 *	+43-1-5055304	     (area-code 1 maybe changed to 201 in 1998) *
 * eMail:								*
 *	sflip@dbai.tuwien.ac.at						*
 * WWW:									*
 *	http://www.dbai.tuwien.ac.at/proj/StarFLIP/			*
 *                                                                      *
 * $Id: domlist.cpp,v 1.4 1997/06/24 17:29:02 reichel Exp $ *
 ***********************************************************************/


#include "domlist.h"
 
/*-------------------------------------------------------------------*/
/* ListOfConstr                                                      */
/*-------------------------------------------------------------------*/
 
ListOfConstr::ListOfConstr(void) : GenList()
{}
ListOfConstr::ListOfConstr(ListOfConstr & const source) : GenList(source)
{}
ListOfConstr::ListOfConstr(SledForConstr & const source) : GenList(source)
{}
//===================================================================
ListOfConstr::~ListOfConstr(void)
{
  clear();
}
//===================================================================
void ListOfConstr::del(void *ptr)
{
  if(ptr!=NULL)
    delete((char *)ptr);
}
//===================================================================
void *ListOfConstr::key(void *info)
{
  return(info);
}
//===================================================================
bool ListOfConstr::look(void *wanted,void *info)
{
  return((bool)(strcmp((char *)wanted,(char *)info)==0));
}
//===================================================================
bool ListOfConstr::cmp(void *ptr1,void *ptr2)
{
  return((bool)(strcmp((char *)ptr1,(char *)ptr2)==0));
}
//===================================================================
void ListOfConstr::cpy(void *&item,void *info)
{
  char *temp;
  int len=strlen((char *)info);
  temp=new(char[len+1]);
  strcpy(temp,(char *)info);
  item=(void *)temp;
}
//=================================================================== 
bool ListOfConstr::rd(ifstream &infile,void *&item)
{
  char		meth[LENGTH_OF_NAMES];
  int		len;
  char 		buffer[MAX_STRING_LEN];

  if(infile>>buffer)
  {
    len=strlen(buffer);
    if (len<MAX_STRING_LEN)
    {
      cpy(item,buffer);
      return(true);
    }
    else 
    {
      strcpy(meth,"ListOfConstr::rd (name to long)");
      len=strlen(meth);
      domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
      return(false);
    }
  }
}

//===================================================================
bool ListOfConstr::wr(ofstream &outfile,void *item)
{
  char		meth[LENGTH_OF_NAMES];
  int		len;

  if(outfile<<(char *)item<<' ')
    return(true);
  {
    strcpy(meth,"ListOfConstr::wr");
    len=strlen(meth);
    domflip_error.set(domflip_error_t::file_write_error_in,len,meth);
    return(false);
  }
}
//=================================================================== 
ListOfConstr &ListOfConstr::operator =(ListOfConstr & const source)
{
  GenList::operator=(source);
  return(*this);
}
//=================================================================== 
ListOfConstr &ListOfConstr::operator =(SledForConstr & const source)
{
  GenList::operator=(source);
  return(*this);
}
//===================================================================  
bool ListOfConstr::operator ==(ListOfConstr & const par)
{
  return(GenList::operator ==(par));
}
//=================================================================== 
bool ListOfConstr::week_equal(ListOfConstr & const par)
{
  return(GenList::week_equal(par));
}
//=================================================================== 
bool ListOfConstr::add(char *term,InfoCopy c)
{
  return(GenList::add(term,c));
}
//=================================================================== 
bool ListOfConstr::add_quick(char *term,InfoCopy c)
{
  return(GenList::add_quick(term,c));
}
//===================================================================
bool ListOfConstr::kick_out(char *term)
{
  return(GenList::kick_out(term));
}
//===================================================================  
char *ListOfConstr::lookup(char *s)
{
  return((char *)get_info(GenList::lookup((void *)s)));
}
//===================================================================


/*-------------------------------------------------------------------*/
/* ListOfInt                                                         */
/*-------------------------------------------------------------------*/
 
ListOfInt::ListOfInt(void) : GenList()
{}
ListOfInt::ListOfInt(ListOfInt & const source) : GenList(source)
{}
ListOfInt::ListOfInt(SledForInt & const source) : GenList(source)
{}
ListOfInt::~ListOfInt(void)
{
  clear();
}
//===================================================================
void ListOfInt::del(void *ptr)
{
  if(ptr!=NULL)
    delete((int *)ptr);
}
//===================================================================
void *ListOfInt::key(void *info)
{
  return(info);
}
//===================================================================
bool ListOfInt::look(void *wanted,void *info)
{
  return((bool)(strcmp((char *)wanted,(char *)info)==0));
}
//===================================================================
bool ListOfInt::cmp(void *ptr1,void *ptr2)
{
  return((bool)(strcmp((char *)ptr1,(char *)ptr2)==0));
}
//===================================================================
void ListOfInt::cpy(void *&item,void *info)
{
 item=info;
 // ((int)item)=((int *)info);
}
//=================================================================== 
bool ListOfInt::rd(ifstream &infile,void *&item)
{
  char		meth[LENGTH_OF_NAMES];
  int		len;
  int		i;

  if(infile>> i)
  {
    item=i;
    return(true);
  }
  else
  {
    strcpy(meth,"ListOfInt::rd");
    len=strlen(meth);
    domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
    return(false);
  }
}

//===================================================================
bool ListOfInt::wr(ofstream &outfile,void *item)
{
  char		meth[LENGTH_OF_NAMES];
  int		len;

  if(outfile<<(int)((int *)(item))<<' ')
    return(true);
  else
  {
    strcpy(meth,"ListOfInt::wr");
    len=strlen(meth);
    domflip_error.set(domflip_error_t::file_write_error_in,len,meth);
    return(false);
  }
}

//=================================================================== 
ListOfInt &ListOfInt::operator =(ListOfInt & const source)
{
  GenList::operator=(source);
  return(*this);
}
//=================================================================== 
ListOfInt &ListOfInt::operator =(SledForInt & const source)
{
  GenList::operator=(source);
  return(*this);
}
//===================================================================  
bool ListOfInt::operator ==(ListOfInt & const par)
{
  return(GenList::operator ==(par));
}
//=================================================================== 
bool ListOfInt::week_equal(ListOfInt & const par)
{
  return(GenList::week_equal(par));
}
//=================================================================== 
bool ListOfInt::add(char *term,InfoCopy c)
{
  return(GenList::add(term,c));
}
//=================================================================== 
bool ListOfInt::add_quick(char *term,InfoCopy c)
{
  return(GenList::add_quick(term,c));
}
//===================================================================
bool ListOfInt::kick_out(char *term)
{
  return(GenList::kick_out(term));
}
//===================================================================  
char *ListOfInt::lookup(char *s)
{
  return((char *)get_info(GenList::lookup((void *)s)));
}
//===================================================================


int SledForInt::get_key()
{
  return( (int ) ((int *)(GenSled::operator()()) ) );
}

