/************************************************************************
 * This library is free software; you can redistribute it and/or	*
 * modify it under the terms of the GNU Library General Public		*
 * License as published by the Free Software Foundation; either		*
 * version 2 of the License, or (at your option) any later version.	*
 * 									*
 * This library is distributed in the hope that it will be useful,	*
 * but WITHOUT ANY WARRANTY; without even the implied warranty of	*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*
 * Library General Public License for more details.			*
 * 									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the		*
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,		*
 * Boston, MA  02111-1307, USA.						*
 * 									*
 * The GNU Library General Public License file is included in		*
 * Online-Documentation							*
 *									*
 * GNU's Homepage - http://www.gnu.ai.mit.edu/home.html			*
 *									*
 * Questions & comments referring to *FLIP++: please contact us:	*
 *									*
 * Snail-Mail:								*
 *	Dr. Wolfgang SLANY						*
 *	Database and Expert System Group (E184-2)     			*
 *	Institute of Information Systems               			*
 *	Vienna University of Technology					*
 *	Paniglgasse 16, A-1040 Vienna, AUSTRIA				*
 * Telephone:								*
 *	+43-1-58801-6123     (area-code 1 maybe changed to 201 in 1998) *
 * FAX:									*
 *	+43-1-5055304	     (area-code 1 maybe changed to 201 in 1998) *
 * eMail:								*
 *	sflip@dbai.tuwien.ac.at						*
 * WWW:									*
 *	http://www.dbai.tuwien.ac.at/proj/StarFLIP/			*
 *                                                                      *
 * $Id: genalg.cpp,v 1.4 1997/06/24 17:29:07 reichel Exp $ *
 ***********************************************************************/


/*****************************************************************************/
/* project			OptFlip++				     */
/*****************************************************************************/
/* by whom			Karl Schott				     */
/* with whom			Wolfgang Slany				     */
/* when				1994/1995				     */
/* what				genetic algorithm			     */
/* why				prak 1					     */
/* how				C++					     */
/*****************************************************************************/

#include<iostream.h>
#include<stdio.h>
#include<stdlib.h>

#include "genalg.h"

int get_random(int number)
{
	if (number <= 1)
		return number;
	long random_number = random();
//	long random_number = random(30000L);
	return ((int)random_number % (number-1))+1;
}

void genalg::grap_one_entry_forward(domain_p &entry,domain_ring_p &grap_pointer)
{
	if (grap_pointer == NULL)
		grap_pointer = actual_generation;

	entry = grap_pointer->object;

	grap_pointer = grap_pointer->next;
};

void genalg::grap_one_entry_backward(domain_p &entry, domain_ring_p &grap_pointer)
{
	if (grap_pointer == NULL)
		grap_pointer = actual_generation;

	entry = grap_pointer->object;

	grap_pointer = grap_pointer->prev;
};

void genalg::grap_two_entries(domain_p &entry1,domain_p &entry2)
{
	if (grap_pointer1 == grap_pointer2)
		grap_one_entry_forward(entry1,grap_pointer1);

	grap_one_entry_backward(entry2,grap_pointer2);
};

void genalg::crossover_two_entries(Domain *entry1, Domain *entry2,
			   Domain *&child1, Domain *&child2)
{
	if (crossover_strategie & POSITION_BASED_CROSSOVER) {
		do_position_based_crossover(entry1,entry2,child1,child2);
	 	// !!!
	}
	
	if (crossover_strategie & ORDER_BASED_CROSSOVER) {
		do_order_based_crossover(entry1,entry2,child1,child2);
		// !!!
	}
};

void genalg::mutate_entry(Domain *&child)
{
	if (mutation_strategie & POSITION_BASED_MUTATION)
		do_position_based_mutation(child);

	if (mutation_strategie & ORDER_BASED_MUTATION)
		do_order_based_mutation(child);
};

void genalg::evaluate_survivers(void)
{
	switch(survival_strategie)
	{
		case BEST_N_PERCENT_SURVIVE:
			do_best_n_percent_survive();
			break;
		case BEST_N_PERCENT_DESCENDANTS_SURVIVE:
			do_best_n_percent_descendants_survive();
			break;
	}
};
		
// Next section: Adults only!

int genalg::do_position_based_crossover(Domain *entry1, Domain *entry2,
				Domain *&child1, Domain *&child2)
{
	ind position_marker;

	int position_tmp_result;

	child1 = new Domain;
	child2 = new Domain;

	if ((child1 == NULL) || (child2 == NULL))
	{
		perror("Out of memory while creating new child for GA-crossover");
		return -1;
	}

	child1->copy(*entry1); //initialize the two new childs
	child2->copy(*entry2);

	// get two positions for position based crossover

	int position1;
	int position2;

	position1 = get_random(entry1->get_total_number_of_entries());
	position2 = get_random(entry2->get_total_number_of_entries());

	// crossover for first child

	position_tmp_result = entry2->get_total_entry(position1);

	position_marker = child1->dom_total_lookup(&position_tmp_result);
	child1->move(position_marker,child1->get_domain_position(position1));

	position_tmp_result = entry2->get_total_entry(position2);

	position_marker = child1->dom_total_lookup(&position_tmp_result);
	child1->move(position_marker,child1->get_domain_position(position2));

	// crossover for second child

	position_tmp_result = entry1->get_total_entry(position1);

	position_marker = child2->dom_total_lookup((void *)&position_tmp_result);
	child2->move(position_marker,child2->get_domain_position(position1));

	position_tmp_result = entry1->get_total_entry(position2);

	position_marker = child2->dom_total_lookup(&position_tmp_result);
	child2->move(position_marker,child2->get_domain_position(position2));

	return 0;
}

int genalg::do_order_based_crossover(Domain *entry1, Domain *entry2,
			     Domain *&child1, Domain *&child2)
{
	child1 = new Domain;
	child2 = new Domain;

	int position_tmp_marker1;
	int position_tmp_marker2;

	if ((child1 == NULL) || (child2 == NULL))
	{
		perror("Out of memory while creating new child for GA-crossover");
		return -1;
	}

	child1->copy(*entry1); //initialize the two new childs
	child2->copy(*entry2);

	// get two positions for positon based crossover

	int position1 = get_random(entry1->get_total_number_of_entries());
	int position2 = get_random(entry2->get_total_number_of_entries());
 
	// crossover for first child

	position_tmp_marker1 = entry2->get_total_entry(position1);
	position_tmp_marker2 = entry2->get_total_entry(position2);

	child1->change(child1->dom_total_lookup(&position_tmp_marker1),
		       child1->dom_total_lookup(&position_tmp_marker2));

	// crossover for second child

	position_tmp_marker1 = entry1->get_total_entry(position1);
	position_tmp_marker2 = entry1->get_total_entry(position2);

	child2->change(child2->dom_total_lookup(&position_tmp_marker1),
		       child2->dom_total_lookup(&position_tmp_marker2));

	return 0;			
};

void genalg::do_position_based_mutation(Domain *child)
{
	int number_of_entries = child->get_total_number_of_entries();

	ind pos1 = child->get_domain_position(get_random(number_of_entries));
	ind pos2 = child->get_domain_position(get_random(number_of_entries));
	
	child->change(pos1,pos2);
};

void genalg::do_order_based_mutation(Domain *child)
{
	int number_of_entries = child->get_total_number_of_entries();

	ind move_from_pos = child->get_domain_position(get_random(number_of_entries));
	ind to_pos = child->get_domain_position(get_random(number_of_entries));

	child->move(move_from_pos,to_pos);
};
		
void genalg::do_best_n_percent_survive(void)
{
	unsigned long int counter = 1;
	unsigned long int maxcounter = ((childs_number + actual_generation_number)*percent_survivers)/100;

	domain_ring_p tmp_actual_generation = NULL;
	domain_ring_p tmp_pointer_adults = NULL;
	domain_ring_p tmp_pointer_childs = NULL;

	unsigned long tmp_actual_generation_number = 0;

	while (counter < maxcounter)
	{
		if ((actual_generation_maxnumber != 0) && 
		    (tmp_actual_generation_number > actual_generation_maxnumber))
			break;

		if (tmp_pointer_adults == NULL)
			get_first_entry(&actual_generation,&tmp_pointer_adults);

		if (tmp_pointer_childs == NULL)
			get_first_entry(&childs,&tmp_pointer_childs);

		if ((tmp_pointer_adults == NULL) && (tmp_pointer_childs == NULL))
			break;
		else if ((tmp_pointer_adults == NULL) && (tmp_pointer_childs != NULL))
		{
			create_new_entry(&tmp_actual_generation,tmp_pointer_childs->object,
					 tmp_actual_generation);
			tmp_pointer_childs = NULL;
		}
		else if ((tmp_pointer_adults != NULL) && (tmp_pointer_childs == NULL))
		{
			create_new_entry(&tmp_actual_generation,tmp_pointer_adults->object,
					 tmp_actual_generation);
			tmp_pointer_adults = NULL;
		}
		else if (tmp_pointer_adults->fitness >= tmp_pointer_childs->fitness)
		{
			create_new_entry(&tmp_actual_generation,tmp_pointer_adults->object,
					 tmp_actual_generation);
			tmp_pointer_adults = NULL;
		}
		else
		{
			create_new_entry(&tmp_actual_generation,tmp_pointer_childs->object,
					 tmp_actual_generation);
			tmp_pointer_childs = NULL;
		}
		
		tmp_actual_generation_number++;

	}
	
	clear_strings();
	
	actual_generation = tmp_actual_generation;

	actual_generation_number = tmp_actual_generation_number;		
};

void genalg::do_best_n_percent_descendants_survive(void)
{
	unsigned long int counter = 1;
	unsigned long int max_counter = (childs_number * percent_survivers)/100;

	domain_ring_p tmp_domain_ring_pointer = childs;

	// Clear the parents

	clear_actual_strings(actual_generation);

	// Count surviving strings

	actual_generation = childs;

	while (counter < max_counter)
	{
		if (tmp_domain_ring_pointer->next == childs)
			break;

		tmp_domain_ring_pointer = tmp_domain_ring_pointer->next;
		counter++;

	}

	// rip up the chain

	if (tmp_domain_ring_pointer->next != childs)
	{
		(*(tmp_domain_ring_pointer->next)).prev = childs->prev;
		(*(childs->prev)).next = tmp_domain_ring_pointer->next;
		childs = tmp_domain_ring_pointer->next;
		clear_actual_strings(childs);
	}
	else
		childs = NULL;

	// tie up the new chain

	(*actual_generation).prev = tmp_domain_ring_pointer;
	(*tmp_domain_ring_pointer).next = actual_generation;
};
		
int genalg::create_new_entry(DOMAIN_RING **domain_ring,Domain *entry,DOMAIN_RING *start_merker,
		     int first = TRUE)
{
	if (first == TRUE)
	{
		if (start_merker == NULL)
			;
		else if ((start_merker == childs) && (childs_maxnumber != 0) &&
			(childs_number >= childs_maxnumber - 1))
			return 0;
		else if ((start_merker == actual_generation) && (actual_generation_maxnumber != 0) &&
			(actual_generation_number >= actual_generation_maxnumber-1))
			return 0;
		//entry->evaluate_plan();
	}

	if ((*domain_ring == NULL) || (entry->get_evaluation() >= (*domain_ring)->fitness) ||
	   ((first == FALSE) && (*domain_ring == start_merker)))
	{
		domain_ring_p tmp_domain_ring = new DOMAIN_RING;
	
		if (tmp_domain_ring == NULL)
			return -1;

		tmp_domain_ring->object = entry;
		tmp_domain_ring->fitness = entry->get_evaluation();

		tmp_domain_ring->next = (*domain_ring != NULL) ? *domain_ring : tmp_domain_ring;
		tmp_domain_ring->prev = (*domain_ring != NULL) ? (*domain_ring)->prev : tmp_domain_ring;

		if (*domain_ring != NULL)
		{
			(*(tmp_domain_ring->next)).prev = tmp_domain_ring;
			(*(tmp_domain_ring->prev)).next = tmp_domain_ring;
		}

		if (first == TRUE)
			*domain_ring = tmp_domain_ring;

		if (start_merker == childs)
			childs_number++;
		else
			actual_generation_number++;

		return 0;

	}
	else
	{
		return create_new_entry(&((**domain_ring).next),entry,start_merker,FALSE);
	}			
};

void genalg::get_first_entry(DOMAIN_RING **tmp_domain_ring,DOMAIN_RING **eintrag)
{
	if (*tmp_domain_ring != NULL)
	{
		*eintrag = *tmp_domain_ring;

		(*((*tmp_domain_ring)->next)).prev = (*tmp_domain_ring)->prev;
		(*((*tmp_domain_ring)->prev)).next = (*tmp_domain_ring)->next;

		if ((*tmp_domain_ring)->next != *tmp_domain_ring)
			*tmp_domain_ring = (*tmp_domain_ring)->next;
		else
			*tmp_domain_ring = NULL;
	}
	else
		*eintrag = NULL;

};


genalg::genalg(void)
{
	actual_generation = NULL;
	childs = NULL;

	actual_generation_number = 0;
	actual_generation_maxnumber = 20;
	childs_number = 0;
	childs_maxnumber = 50;

	max_init_mixture = 20;
	percent_survivers=100;

	mutation_strategie = POSITION_BASED_MUTATION;
	crossover_strategie = POSITION_BASED_CROSSOVER;
	survival_strategie = BEST_N_PERCENT_SURVIVE;
	
	do_repair = FALSE;

};

genalg::~genalg(void)
{
	clear_strings();
};

int genalg::set_crossover(unsigned short crossover)
{
	if (crossover > LAST_CROSSOVER_VALUE)
		return -1;

	crossover_strategie = crossover;

	return 0;
};

int genalg::set_mutation(unsigned short mutation)
{
	if (mutation > LAST_MUTATION_VALUE)
		return -1;

	mutation_strategie = mutation;

	return 0;
};

int genalg::set_strategie(genalg_strategie strategie)
{
	survival_strategie = strategie;

	return 0;
};

void genalg::set_percent_survivers(unsigned short percent_survivers)
{
	if ((percent_survivers <= 100) || (percent_survivers > 0))
		this->percent_survivers = percent_survivers;
};

void genalg::set_max_strings_number(unsigned long maxstring)
{
	actual_generation_maxnumber = maxstring;
};

void genalg::set_max_childs_number(unsigned long maxchilds)
{
	childs_maxnumber = maxchilds;
};

void genalg::set_max_init_mixture(unsigned int init_mixture)
{
	max_init_mixture = init_mixture;
};

void genalg::set_repair(unsigned int bool)
{
	do_repair = bool;
};

void genalg::clear_actual_strings(domain_ring_p &tmp_domain_ring)
{
	domain_ring_p domain_ring_stepper = tmp_domain_ring;
	domain_ring_p domain_ring_marker;

	if (tmp_domain_ring == childs)
		childs_number = 0;
	else
		actual_generation_number = 0;

	if (tmp_domain_ring != NULL)
		do {
			domain_ring_marker = domain_ring_stepper->next;
			// delete domain_ring_stepper->object;
			// delete domain_ring_stepper;
			domain_ring_stepper = domain_ring_marker;
		} while (domain_ring_stepper != tmp_domain_ring);

	tmp_domain_ring = NULL;

};

int genalg::clear_strings(void)
{
	if (actual_generation != NULL)
		clear_actual_strings(actual_generation);

	if (childs != NULL)
		clear_actual_strings(childs);

	return 0;
};

int genalg::set_string(Domain *my_domain)
{
	return create_new_entry(&actual_generation,my_domain,actual_generation);
};	

int genalg::set_string_copy(Domain my_domain)
{
	Domain *tmp_domain_obj = new Domain;
		
	tmp_domain_obj->copy(my_domain);

	return create_new_entry(&actual_generation,tmp_domain_obj,actual_generation);
};

int genalg::init_start_strings(void)
{
	Domain *tmp_domain;
	int nr_of_entries;
	int tmp_max_init_mixture;

	if (actual_generation->object == NULL)
		return -1;


	for (int i = 0; i < 10;i++)
	{
		tmp_domain	= new Domain;

		tmp_domain->copy(*(actual_generation->object));

		nr_of_entries = tmp_domain->get_nr_of_entries();

		tmp_max_init_mixture = get_random(max_init_mixture);

		for (int i = 0; i < tmp_max_init_mixture; i++)
			tmp_domain->change(tmp_domain->get_domain_position(get_random(nr_of_entries)),
					   tmp_domain->get_domain_position(get_random(nr_of_entries)));			

		set_string(tmp_domain);
	}
	return 0;
};

void genalg::repair(Domain *b_shed)
{
  SledForCviol 		sfcv = b_shed->get_c_violl();  
  int tmp;

 	 int                  h = get_random(sfcv.get_how_full());
 	 ConstrViolation *    cv =  ((ConstrViolation*)sfcv.operator[](h));
 	 Repair *             r = b_shed->get_repairl()->lookup(cv->get_name());
	 SledForInt    	sfi=*(r->get_rep_steps());
	 int                  i = get_random(sfi.get_how_full());

         tmp=b_shed->get_aggs()->position();

      if( b_shed->get_advice(i,h))
      {
        if(b_shed->call_repair(h))
        {
	  b_shed->get_aggs()->operator[](tmp);
        }
      }
      b_shed->get_aggs()->operator[](tmp);
}

float genalg::one_step(void)
{
	Domain *entry1,*entry2,*child1,*child2;

	grap_pointer1 = NULL;
	grap_pointer2 = NULL;

	// Unterneath the mean GA routine

	for (unsigned long i = 0; i < childs_maxnumber; i++)
	{
		grap_two_entries(entry1,entry2);
		crossover_two_entries(entry1,entry2,child1,child2);
		mutate_entry(child1);
		mutate_entry(child2);
		child1->evaluate_plan();
		child2->evaluate_plan();
		if (do_repair == TRUE)
		{
			repair(child1);
			repair(child2);	
			child1->evaluate_plan();
			child2->evaluate_plan();
		}
		create_new_entry(&childs,child1,childs);
		create_new_entry(&childs,child2,childs);
	}

	// print_all_child_scheds();

	evaluate_survivers();

	return actual_generation->fitness;
};

float genalg::several_steps(unsigned long steps)
{
	float tmp_fitness;

	for (unsigned long i = 0; i < steps; i++)
	{
		cout << i << endl;
		tmp_fitness = one_step();
	}

	return tmp_fitness;
};

float genalg::go_to_fitness(float fitness, unsigned long maxsteps = 0L)
{
	float tmp_fitness;
	unsigned long steps_counter = 0L;

	do {
		tmp_fitness = one_step();
		if ((++steps_counter >= maxsteps) && (maxsteps != 0))
			break;
	} while (tmp_fitness < fitness);

	return tmp_fitness;
};

float genalg::get_string_with_hightest_fitness(Domain *&fitest_domain)
{
	if (actual_generation != NULL)
	{
		fitest_domain = actual_generation->object;
		return actual_generation->fitness;
	}
	else
	{
		fitest_domain = NULL;
		return 0;
	}
};

float genalg::get_string_with_hightest_fitness_copy(Domain &fitest_domain)
{
	if (actual_generation != NULL)
	{
		fitest_domain.copy(*(actual_generation->object));
		return actual_generation->fitness;
	}
	else
		return 0;
};

void genalg::print_all_acutal_scheds(void)
{
	domain_ring_p tmp_ring_p = actual_generation;

	if (tmp_ring_p != NULL)
	{
		do {
			tmp_ring_p->object->show_aggregats();
			tmp_ring_p = tmp_ring_p->next;
		} while (tmp_ring_p != actual_generation);
	}
	else
		cout << "Nothing in actual generation\n";
};

void genalg::print_all_child_scheds(void)
{
	domain_ring_p tmp_ring_p = childs;

	if (tmp_ring_p != NULL)
	{
		do {
			tmp_ring_p->object->show_aggregats();
			tmp_ring_p = tmp_ring_p->next;
		} while (tmp_ring_p != childs);
	}
	else
		cout << "Nothing in childs\n";

};

void genalg::set_param(genalg_param param)
{
	set_max_strings_number(param.actual_generation_maxnumber);
	set_max_childs_number(param.childs_maxnumber);

	set_percent_survivers(param.percent_survivers);
	set_max_init_mixture(param.max_init_mixture);

	set_repair(param.do_repair);

	set_mutation(param.mutation_strategie);
	set_crossover(param.crossover_strategie);
	set_strategie(param.survival_strategie);
}
void genalg::get_param(genalg_param &param)
{
	param.actual_generation_number = actual_generation_number;
	param.actual_generation_maxnumber = actual_generation_maxnumber;
	param.childs_number = childs_number;
	param.childs_maxnumber = childs_maxnumber;

	param.percent_survivers = percent_survivers;
	param.max_init_mixture = max_init_mixture;

	param.do_repair = do_repair;

	param.mutation_strategie = mutation_strategie;
	param.crossover_strategie = crossover_strategie;
	param.survival_strategie = survival_strategie;
}



