/************************************************************************
 * This library is free software; you can redistribute it and/or	*
 * modify it under the terms of the GNU Library General Public		*
 * License as published by the Free Software Foundation; either		*
 * version 2 of the License, or (at your option) any later version.	*
 * 									*
 * This library is distributed in the hope that it will be useful,	*
 * but WITHOUT ANY WARRANTY; without even the implied warranty of	*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*
 * Library General Public License for more details.			*
 * 									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the		*
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,		*
 * Boston, MA  02111-1307, USA.						*
 * 									*
 * The GNU Library General Public License file is included in		*
 * Online-Documentation							*
 *									*
 * GNU's Homepage - http://www.gnu.ai.mit.edu/home.html			*
 *									*
 * Questions & comments referring to *FLIP++: please contact us:	*
 *									*
 * Snail-Mail:								*
 *	Dr. Wolfgang SLANY						*
 *	Database and Expert System Group (E184-2)     			*
 *	Institute of Information Systems               			*
 *	Vienna University of Technology					*
 *	Paniglgasse 16, A-1040 Vienna, AUSTRIA				*
 * Telephone:								*
 *	+43-1-58801-6123     (area-code 1 maybe changed to 201 in 1998) *
 * FAX:									*
 *	+43-1-5055304	     (area-code 1 maybe changed to 201 in 1998) *
 * eMail:								*
 *	sflip@dbai.tuwien.ac.at						*
 * WWW:									*
 *	http://www.dbai.tuwien.ac.at/proj/StarFLIP/			*
 *                                                                      *
 * $Id: genalg.h,v 1.4 1997/06/24 17:29:08 reichel Exp $ *
 ***********************************************************************/


/*****************************************************************************/
/* project			OptiFlip++				     */
/*****************************************************************************/
/* by whom			Karl Schott				     */
/* with whom			Wolfgang Slany				     */
/* when				1994/1995				     */
/* what				genetic algorithm			     */
/* why				prak 1					     */
/* how				C++					     */
/*****************************************************************************/


#ifndef _GENALG_OBJECT_HPP_
#define _GENALG_OBJECT_HPP_

#include "hash.h"

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

// Constants for the set_mutation-function, to allow you
// to let the GA doing a specific mutation.
// The values can be combined.

unsigned short const POSITION_BASED_MUTATION = 0x01;
unsigned short const ORDER_BASED_MUTATION = 0x02;
// unsigned short const SWAP_MUTATION = 0x04;
// unsigned short const SPLICE_MUTATION = 0x08;
// unsigned short const INVERSION_MUTATION = 0x10;
unsigned short const LAST_MUTATION_VALUE = ORDER_BASED_MUTATION*2 - 1;
unsigned short const NUMBER_OF_MUTATIONS = 2;

// Constants for the set_crossover-function, to control the crossover
// the GA uses. Values can be combined.

unsigned short const POSITION_BASED_CROSSOVER = 0x01;
unsigned short const ORDER_BASED_CROSSOVER = 0x02;
// unsigned short const CYCLE_CROSSOVER = 0x04;
// unsigned short const PARTIALLY_MAPPED_CROSSOVER = 0x08;
// unsigned short const INTERSECTION_AND_UNION = 0x10;
unsigned short const LAST_CROSSOVER_VALUE = ORDER_BASED_CROSSOVER*2 - 1;
unsigned short const NUMBER_OF_CROSSOVER = 2;

// Enumeration type to control the survival strategie the GA uses.

typedef enum genalg_strategie {
		BEST_N_PERCENT_SURVIVE = 0,
		BEST_N_PERCENT_DESCENDANTS_SURVIVE = 1
		// PROPORTIONAL_FITNESS_SURVIVAL = 2,
		// ELITISM = 3
		};

struct genalg_param {
		unsigned long		actual_generation_number;
		unsigned long		actual_generation_maxnumber;
		unsigned long		childs_number;
		unsigned long		childs_maxnumber;

		unsigned long		percent_survivers;
		unsigned int		max_init_mixture;

		unsigned int		do_repair;

		unsigned short		mutation_strategie;
		unsigned short		crossover_strategie;
		genalg_strategie	survival_strategie;
	} GENALG_PARAM;

class genalg
{
	protected:
		// GA-Strings
		typedef struct domain_ring {
			Domain 		*object; // Schedule
			float   	fitness; // Fitness of the Schedule
			domain_ring	*next;	 // Next element
			domain_ring	*prev;	 // Previous element
			} DOMAIN_RING;

		typedef Domain		*domain_p;	// Just two pointers
		typedef domain_ring 	*domain_ring_p; // to make function
							// typing shorter.

		// Parents
		domain_ring		*actual_generation;
		// Childs
		domain_ring		*childs;
		
		domain_ring		*grap_pointer1;
		domain_ring		*grap_pointer2;

		unsigned long		actual_generation_number;
		unsigned long		actual_generation_maxnumber;
		unsigned long		childs_number;
		unsigned long		childs_maxnumber;

		unsigned long		percent_survivers;
		unsigned int		max_init_mixture;

		unsigned int		do_repair;

		// controll-registers for the GA-methods

		unsigned short		mutation_strategie;
		unsigned short		crossover_strategie;
		genalg_strategie	survival_strategie;

		void grap_one_entry_forward(domain_p &entry,domain_ring_p &grap_pointer);
		void grap_one_entry_backward(domain_p &entry, domain_ring_p &grap_pointer);
		void grap_two_entries(domain_p &entry1,domain_p &entry2);
		void crossover_two_entries(Domain *entry1, Domain *entry2,
					   Domain *&child1, Domain *&child2);
		void mutate_entry(Domain *&child);
		void evaluate_survivers(void);
		
		int do_position_based_crossover(Domain *entry1, Domain *entry2,
						Domain *&child1, Domain *&child2);
		int do_order_based_crossover(Domain *entry1, Domain *entry2,
					     Domain *&child1, Domain *&child2);

		void do_position_based_mutation(Domain *child);
		void do_order_based_mutation(Domain *child);
		
		void do_best_n_percent_survive(void);
		void do_best_n_percent_descendants_survive(void);
		
		int create_new_entry(DOMAIN_RING **domain_ring,Domain *entry,DOMAIN_RING *start_merker,
				     int first = TRUE);

		void get_first_entry(DOMAIN_RING **temp_domain_ring,DOMAIN_RING **eintrag);
		void repair(Domain *b_shed);


	public:
		genalg(void);
		~genalg(void);

		int set_crossover(unsigned short crossover);
		int set_mutation(unsigned short mutation);
		int set_strategie(genalg_strategie strategie);
		void set_percent_survivers(unsigned short percent_survivers);
		void set_max_strings_number(unsigned long maxstring);
		void set_max_childs_number(unsigned long maxchilds);
		void set_max_init_mixture(unsigned int init_mixture);
		void set_repair(unsigned int bool);

		void clear_actual_strings(domain_ring_p &tmp_domain_ring);
		int clear_strings(void);
		int set_string(Domain *my_domain);	// Insert a new schedule direct
		int set_string_copy(Domain my_domain);	// Insert a new schedule by copying the source

		int init_start_strings(void);

		float one_step(void);
		float several_steps(unsigned long steps);
		float go_to_fitness(float fitness, unsigned long maxsteps = 0L);

		float get_string_with_hightest_fitness(Domain *&fitest_domain);
		float get_string_with_hightest_fitness_copy(Domain &fitest_domain);

		void set_param(genalg_param param);
		void get_param(genalg_param &param);

		void print_all_acutal_scheds(void); // For test purposes only
		void print_all_child_scheds(void);
	};

#endif
			
