/************************************************************************
 * This library is free software; you can redistribute it and/or	*
 * modify it under the terms of the GNU Library General Public		*
 * License as published by the Free Software Foundation; either		*
 * version 2 of the License, or (at your option) any later version.	*
 * 									*
 * This library is distributed in the hope that it will be useful,	*
 * but WITHOUT ANY WARRANTY; without even the implied warranty of	*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*
 * Library General Public License for more details.			*
 * 									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the		*
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,		*
 * Boston, MA  02111-1307, USA.						*
 * 									*
 * The GNU Library General Public License file is included in		*
 * Online-Documentation							*
 *									*
 * GNU's Homepage - http://www.gnu.ai.mit.edu/home.html			*
 *									*
 * Questions & comments referring to *FLIP++: please contact us:	*
 *									*
 * Snail-Mail:								*
 *	Dr. Wolfgang SLANY						*
 *	Database and Expert System Group (E184-2)     			*
 *	Institute of Information Systems               			*
 *	Vienna University of Technology					*
 *	Paniglgasse 16, A-1040 Vienna, AUSTRIA				*
 * Telephone:								*
 *	+43-1-58801-6123     (area-code 1 maybe changed to 201 in 1998) *
 * FAX:									*
 *	+43-1-5055304	     (area-code 1 maybe changed to 201 in 1998) *
 * eMail:								*
 *	sflip@dbai.tuwien.ac.at						*
 * WWW:									*
 *	http://www.dbai.tuwien.ac.at/proj/StarFLIP/			*
 *                                                                      *
 * $Id: order.cpp,v 1.4 1997/06/24 17:29:13 reichel Exp $ *
 ***********************************************************************/


#include <fstream.h>
#include "order.h"


//-------------------------------------------------------------------------
Order::Order(void)			// Order constructor #1
{
  init();
}
//-------------------------------------------------------------------------
Order::Order(Order  & const source)
{
  init();
  copy(source);
}
//-------------------------------------------------------------------------
virtual Order::~Order(void)	// Order destructor #1
{
};

//-------------------------------------------------------------------------
void Order::init(void)
{
  onumber=0;
  strcpy(plantmark,"");
  weight=0;
  item_constr=new(ListOfConstr);
}
//-------------------------------------------------------------------------
void Order::clear(void)
{
  item_constr.clear();
}
//-------------------------------------------------------------------------
void Order::copy(Order & const source)
{ 
  onumber = source.onumber;
  strcpy(plantmark,source.plantmark);
  weight = source.weight;
  item_constr =source.item_constr;
}

//-------------------------------------------------------------------------
Order &Order::operator = (Order & const source)
{
  clear();
  copy(source);
  return(*this);
};

//===================================================================
bool Order::write(ofstream &outfile,bool up)
{
  int 		k;
  char		meth[LENGTH_OF_NAMES];
  int		len;


  if(!(outfile<<endl<<"    "<<onumber<<" "<<plantmark<<" "<<weight<<endl<<"       "))
  {
    strcpy(meth,"Order::write");
    len=strlen(meth);
    domflip_error.set(domflip_error_t::file_write_error_in,len,meth);
    return(false);
  }
  else
  {
    if (!item_constr.write(outfile))
    {
      strcpy(meth,"Order::write item_constr");
      len=strlen(meth);
      domflip_error.set(domflip_error_t::file_write_error_in,len,meth);
      return(false);
    }
  }
  return(true);
}

//===================================================================
bool Order::read (ifstream &infile,bool up)
{
  char		meth[LENGTH_OF_NAMES];
  int		len;

  if(up)
    if(!Order::read(infile))
    {
      strcpy(meth,"Order::read");
      len=strlen(meth);
      domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
      return(false);
    }
  if(!(infile>>onumber))
  {
    strcpy(meth,"Order::read (onumber)");
    len=strlen(meth);
    domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
    // clear();
    return(false);
  }
  else
  {
    if(!(infile>>plantmark))
    {
      strcpy(meth,"Order::read (plantmark)");
      len=strlen(meth);
      domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
      return(false);
    }
    else
    {
      if(!(infile>>weight))
      {
        strcpy(meth,"Order::read (weight)");
        len=strlen(meth);
        domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
        return(false);
      }

      else
      {
    	if(!item_constr.read(infile))
      	  return (false);
      }

    }
  }
  return(true);
};
/*   
//-------------------------------------------------------------------------

void Order::set_onumber(long nr)
{
  onumber = nr;
}
*/
long Order::get_onumber()
{
  return(onumber);
}

//-------------------------------------------------------------------------
int Order::get_key()
{
  return(onumber % 32767);
}
//-------------------------------------------------------------------------
/*
void Order::set_plantmark(char *pm)
{
  strcpy(plantmark,pm);
}
*/
char* Order::get_plantmark(void)
{
  return(plantmark);
}
//-------------------------------------------------------------------------
/*
void Order::set_weight(long we)
{
  weight = we;
}
*/
//-------------------------------------------------------------------------
long Order::get_weight()
{
  return(weight);
}




