/************************************************************************
 * This library is free software; you can redistribute it and/or	*
 * modify it under the terms of the GNU Library General Public		*
 * License as published by the Free Software Foundation; either		*
 * version 2 of the License, or (at your option) any later version.	*
 * 									*
 * This library is distributed in the hope that it will be useful,	*
 * but WITHOUT ANY WARRANTY; without even the implied warranty of	*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*
 * Library General Public License for more details.			*
 * 									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the		*
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,		*
 * Boston, MA  02111-1307, USA.						*
 * 									*
 * The GNU Library General Public License file is included in		*
 * Online-Documentation							*
 *									*
 * GNU's Homepage - http://www.gnu.ai.mit.edu/home.html			*
 *									*
 * Questions & comments referring to *FLIP++: please contact us:	*
 *									*
 * Snail-Mail:								*
 *	Dr. Wolfgang SLANY						*
 *	Database and Expert System Group (E184-2)     			*
 *	Institute of Information Systems               			*
 *	Vienna University of Technology					*
 *	Paniglgasse 16, A-1040 Vienna, AUSTRIA				*
 * Telephone:								*
 *	+43-1-58801-6123     (area-code 1 maybe changed to 201 in 1998) *
 * FAX:									*
 *	+43-1-5055304	     (area-code 1 maybe changed to 201 in 1998) *
 * eMail:								*
 *	sflip@dbai.tuwien.ac.at						*
 * WWW:									*
 *	http://www.dbai.tuwien.ac.at/proj/StarFLIP/			*
 *                                                                      *
 * $Id: repair.cpp,v 1.4 1997/06/24 17:29:14 reichel Exp $ *
 ***********************************************************************/


#include <fstream.h>
#include "repair.h"



//-------------------------------------------------------------------------
Repair::Repair(void)
{
  init();
}
//-------------------------------------------------------------------------
Repair::Repair(Repair  & const source)
{
  init();
  copy(source);
}
//-------------------------------------------------------------------------
virtual Repair::~Repair(void)	// Repair destructor #1
{};
//-------------------------------------------------------------------------
void Repair::init(void)
{
  strcpy(name,"");
  rep_steps=new(ListOfInt);
}
//-------------------------------------------------------------------------
void Repair::clear(void)
{
  strcpy(name,"");
  rep_steps=new(ListOfInt);
}
//-------------------------------------------------------------------------
void Repair::copy(Repair & const source)
{ 
  strcpy(name,source.name);
  rep_steps=source.rep_steps;
}

//-------------------------------------------------------------------------
Repair &Repair::operator = (Repair & const source)
{
  clear();
  copy(source);
  return(*this);
};

//===================================================================
bool Repair::write(ofstream &outfile,bool up)
{
  char		meth[LENGTH_OF_NAMES];
  int		len;
  int 		k;

  if(!(outfile<<endl<<"  "<<name<<" "<<endl<<"       "))
  {
    strcpy(meth,"Repair::write (name)");
    len=strlen(meth);
    domflip_error.set(domflip_error_t::file_write_error_in,len,meth);
    return(false);
  }
  else
  {
    if (!rep_steps.write(outfile))
    {
      domflip_error.set(domflip_error_t::file_write_error_in,len,meth);	// ???
      return(false);
    }
    else
     return(true);   
  }
}

//===================================================================
bool Repair::read (ifstream &infile,bool up)
{
  char		meth[LENGTH_OF_NAMES];
  int		len;

  if(up)
    if(!Repair::read(infile))
    {
      strcpy(meth,"Repair::read");
      len=strlen(meth);
      domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
      clear();
      return(false);
    }

  if(!(infile>>name))
  {
    strcpy(meth,"Repair::read (name)");
    len=strlen(meth);
    domflip_error.set(domflip_error_t::file_read_error_in,len,meth);
    clear();
    return(false);
  }
  else
  {
    if(!(rep_steps.read(infile)))
    {
      clear();
      return(false);
    }
  }
  return(true);
};
    

//-------------------------------------------------------------------------
int Repair::get_key(int i)	// !!!!
{
  SledForInt sfi;

  sfi=rep_steps;
  sfi.operator[](i);

  return(sfi.get_key());
}
//-------------------------------------------------------------------------
char* Repair::get_name(void)
{
  return(name);
}

//-------------------------------------------------------------------------



ListOfInt* Repair::get_rep_steps()
{
  return(&rep_steps);
}

